;;;; crypt.scm -- bindings to cryptlib
;;;; Copyright (c) 2004 by Thomas Christian Chust <chust@web.de>.
;;;; All rights reserved.
;;;;
;;;; Redistribution and use in source and binary forms, with or
;;;; without modification, are permitted provided that the following
;;;; conditions are met:
;;;;
;;;; * Redistributions of source code must retain the above copyright
;;;;   notice, this list of conditions and the following disclaimer.
;;;; * Redistributions in binary form must reproduce the above
;;;;   copyright notice, this list of conditions and the following
;;;;   disclaimer in the documentation and/or other materials provided
;;;;   with the distribution.
;;;; * Neither the name of the author nor the names of its
;;;;   contributors may be used to endorse or promote products derived
;;;;   from this software without specific prior written permission.
;;;;
;;;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
;;;; CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
;;;; INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;;;; MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;;;; DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
;;;; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
;;;; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
;;;; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
;;;; USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
;;;; AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
;;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
;;;; ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
;;;; POSSIBILITY OF SUCH DAMAGE.

;;;; cryptlib is a high level cryptography library written by Peter Gutmann
;;;; you can find information and downloads for it at
;;;;   http://www.cs.auckland.ac.nz/~pgut001/cryptlib/
;;;; At the time of writing of this code, the library is available
;;;; under the same license conditions as this code

;;;; Modified for cryptlib 3.2.2, Kon Lovett, Sep 28 2005
;;;; Note: Distributed software that uses cryptlib must include
;;;; credit to cryptlib's authors.

(cond-expand
 ((or chicken-compile-shared shared)
  )
 (else
  (declare (unit crypt))))

(declare
 (usual-integrations)
 (fixnum-arithmetic)
 (uses srfi-4 lolevel)
 (hide data-byte-length)
 (export CRYPT-ALGO-NONE
	 CRYPT-ALGO-DES
	 CRYPT-ALGO-3DES
	 CRYPT-ALGO-IDEA
	 CRYPT-ALGO-CAST
	 CRYPT-ALGO-RC2
	 CRYPT-ALGO-RC4
	 CRYPT-ALGO-RC5
	 CRYPT-ALGO-AES
	 CRYPT-ALGO-BLOWFISH
	 CRYPT-ALGO-SKIPJACK
	 CRYPT-ALGO-DH
	 CRYPT-ALGO-RSA
	 CRYPT-ALGO-DSA
	 CRYPT-ALGO-ELGAMAL
	 CRYPT-ALGO-KEA
	 CRYPT-ALGO-MD2
	 CRYPT-ALGO-MD4
	 CRYPT-ALGO-MD5
	 CRYPT-ALGO-SHA
	 CRYPT-ALGO-RIPEMD160
	 CRYPT-ALGO-HMAC-MD5
	 CRYPT-ALGO-HMAC-SHA
	 CRYPT-ALGO-HMAC-RIPEMD160
	 CRYPT-ALGO-LAST
	 CRYPT-ALGO-FIRST-CONVENTIONAL
	 CRYPT-ALGO-LAST-CONVENTIONAL
	 CRYPT-ALGO-FIRST-PKC
	 CRYPT-ALGO-LAST-PKC
	 CRYPT-ALGO-FIRST-HASH
	 CRYPT-ALGO-LAST-HASH
	 CRYPT-ALGO-FIRST-MAC
	 CRYPT-ALGO-LAST-MAC
	 CRYPT-MODE-NONE
	 CRYPT-MODE-ECB
	 CRYPT-MODE-CBC
	 CRYPT-MODE-CFB
	 CRYPT-MODE-OFB
	 CRYPT-MODE-LAST
	 CRYPT-KEYSET-NONE
	 CRYPT-KEYSET-FILE
	 CRYPT-KEYSET-HTTP
	 CRYPT-KEYSET-LDAP
	 CRYPT-KEYSET-ODBC
	 CRYPT-KEYSET-DATABASE
	 CRYPT-KEYSET-PLUGIN
	 CRYPT-KEYSET-ODBC-STORE
	 CRYPT-KEYSET-DATABASE-STORE
	 CRYPT-KEYSET-PLUGIN-STORE
	 CRYPT-KEYSET-LAST
	 CRYPT-DEVICE-NONE
	 CRYPT-DEVICE-FORTEZZA
	 CRYPT-DEVICE-PKCS11
	 CRYPT-DEVICE-CRYPTOAPI
	 CRYPT-DEVICE-LAST
	 CRYPT-CERTTYPE-NONE
	 CRYPT-CERTTYPE-CERTIFICATE
	 CRYPT-CERTTYPE-ATTRIBUTE-CERT
	 CRYPT-CERTTYPE-CERTCHAIN
	 CRYPT-CERTTYPE-CERTREQUEST
	 CRYPT-CERTTYPE-REQUEST-CERT
	 CRYPT-CERTTYPE-REQUEST-REVOCATION
	 CRYPT-CERTTYPE-CRL
	 CRYPT-CERTTYPE-CMS-ATTRIBUTES
	 CRYPT-CERTTYPE-RTCS-REQUEST
	 CRYPT-CERTTYPE-RTCS-RESPONSE
	 CRYPT-CERTTYPE-OCSP-REQUEST
	 CRYPT-CERTTYPE-OCSP-RESPONSE
	 CRYPT-CERTTYPE-PKIUSER
	 CRYPT-CERTTYPE-LAST
	 CRYPT-FORMAT-NONE
	 CRYPT-FORMAT-AUTO
	 CRYPT-FORMAT-CRYPTLIB
	 CRYPT-FORMAT-CMS
	 CRYPT-FORMAT-PKCS7
	 CRYPT-FORMAT-SMIME
	 CRYPT-FORMAT-PGP
	 CRYPT-FORMAT-LAST
	 CRYPT-SESSION-NONE
	 CRYPT-SESSION-SSH
	 CRYPT-SESSION-SSH-SERVER
	 CRYPT-SESSION-SSL
	 CRYPT-SESSION-SSL-SERVER
	 CRYPT-SESSION-RTCS
	 CRYPT-SESSION-RTCS-SERVER
	 CRYPT-SESSION-OCSP
	 CRYPT-SESSION-OCSP-SERVER
	 CRYPT-SESSION-TSP
	 CRYPT-SESSION-TSP-SERVER
	 CRYPT-SESSION-CMP
	 CRYPT-SESSION-CMP-SERVER
	 CRYPT-SESSION-SCEP
	 CRYPT-SESSION-SCEP-SERVER
	 CRYPT-SESSION-LAST
	 CRYPT-USER-NONE
	 CRYPT-USER-NORMAL
	 CRYPT-USER-SO
	 CRYPT-USER-CA
	 CRYPT-USER-LAST
	 CRYPT-ATTRIBUTE-NONE
	 CRYPT-PROPERTY-HIGHSECURITY
	 CRYPT-PROPERTY-OWNER
	 CRYPT-PROPERTY-FORWARDCOUNT
	 CRYPT-PROPERTY-LOCKED
	 CRYPT-PROPERTY-USAGECOUNT
	 CRYPT-PROPERTY-NONEXPORTABLE
	 CRYPT-ATTRIBUTE-ERRORTYPE
	 CRYPT-ATTRIBUTE-ERRORLOCUS
	 CRYPT-ATTRIBUTE-INT-ERRORCODE
	 CRYPT-ATTRIBUTE-INT-ERRORMESSAGE
	 CRYPT-ATTRIBUTE-BUFFERSIZE
	 CRYPT-OPTION-INFO-DESCRIPTION
	 CRYPT-OPTION-INFO-COPYRIGHT
	 CRYPT-OPTION-INFO-MAJORVERSION
	 CRYPT-OPTION-INFO-MINORVERSION
	 CRYPT-OPTION-INFO-STEPPING
	 CRYPT-OPTION-ENCR-ALGO
	 CRYPT-OPTION-ENCR-HASH
	 CRYPT-OPTION-ENCR-MAC
	 CRYPT-OPTION-PKC-ALGO
	 CRYPT-OPTION-PKC-KEYSIZE
	 CRYPT-OPTION-SIG-ALGO
	 CRYPT-OPTION-SIG-KEYSIZE
	 CRYPT-OPTION-KEYING-ALGO
	 CRYPT-OPTION-KEYING-ITERATIONS
	 CRYPT-OPTION-CERT-SIGNUNRECOGNISEDATTRIBUTES
	 CRYPT-OPTION-CERT-VALIDITY
	 CRYPT-OPTION-CERT-UPDATEINTERVAL
	 CRYPT-OPTION-CERT-COMPLIANCELEVEL
	 CRYPT-OPTION-CMS-DEFAULTATTRIBUTES
	 CRYPT-OPTION-SMIME-DEFAULTATTRIBUTES
	 CRYPT-OPTION-KEYS-LDAP-OBJECTCLASS
	 CRYPT-OPTION-KEYS-LDAP-OBJECTTYPE
	 CRYPT-OPTION-KEYS-LDAP-FILTER
	 CRYPT-OPTION-KEYS-LDAP-CACERTNAME
	 CRYPT-OPTION-KEYS-LDAP-CERTNAME
	 CRYPT-OPTION-KEYS-LDAP-CRLNAME
	 CRYPT-OPTION-KEYS-LDAP-EMAILNAME
	 CRYPT-OPTION-DEVICE-PKCS11-DVR01
	 CRYPT-OPTION-DEVICE-PKCS11-DVR02
	 CRYPT-OPTION-DEVICE-PKCS11-DVR03
	 CRYPT-OPTION-DEVICE-PKCS11-DVR04
	 CRYPT-OPTION-DEVICE-PKCS11-DVR05
	 CRYPT-OPTION-DEVICE-PKCS11-HARDWAREONLY
	 CRYPT-OPTION-NET-SOCKS-SERVER
	 CRYPT-OPTION-NET-SOCKS-USERNAME
	 CRYPT-OPTION-NET-HTTP-PROXY
	 CRYPT-OPTION-NET-CONNECTTIMEOUT
	 CRYPT-OPTION-NET-READTIMEOUT
	 CRYPT-OPTION-NET-WRITETIMEOUT
	 CRYPT-OPTION-MISC-ASYNCINIT
	 CRYPT-OPTION-MISC-SIDECHANNELPROTECTION
	 CRYPT-OPTION-CONFIGCHANGED
	 CRYPT-OPTION-SELFTESTOK
	 CRYPT-CTXINFO-ALGO
	 CRYPT-CTXINFO-MODE
	 CRYPT-CTXINFO-NAME-ALGO
	 CRYPT-CTXINFO-NAME-MODE
	 CRYPT-CTXINFO-KEYSIZE
	 CRYPT-CTXINFO-BLOCKSIZE
	 CRYPT-CTXINFO-IVSIZE
	 CRYPT-CTXINFO-KEYING-ALGO
	 CRYPT-CTXINFO-KEYING-ITERATIONS
	 CRYPT-CTXINFO-KEYING-SALT
	 CRYPT-CTXINFO-KEYING-VALUE
	 CRYPT-CTXINFO-KEY
	 CRYPT-CTXINFO-KEY-COMPONENTS
	 CRYPT-CTXINFO-IV
	 CRYPT-CTXINFO-HASHVALUE
	 CRYPT-CTXINFO-LABEL
	 CRYPT-CERTINFO-SELFSIGNED
	 CRYPT-CERTINFO-IMMUTABLE
	 CRYPT-CERTINFO-XYZZY
	 CRYPT-CERTINFO-CERTTYPE
	 CRYPT-CERTINFO-FINGERPRINT
	 CRYPT-CERTINFO-FINGERPRINT-MD5
	 CRYPT-CERTINFO-FINGERPRINT-SHA
	 CRYPT-CERTINFO-CURRENT-CERTIFICATE
	 CRYPT-CERTINFO-CURRENT-EXTENSION
	 CRYPT-CERTINFO-CURRENT-FIELD
	 CRYPT-CERTINFO-CURRENT-COMPONENT
	 CRYPT-CERTINFO-TRUSTED-USAGE
	 CRYPT-CERTINFO-TRUSTED-IMPLICIT
	 CRYPT-CERTINFO-SIGNATURELEVEL
	 CRYPT-CERTINFO-VERSION
	 CRYPT-CERTINFO-SERIALNUMBER
	 CRYPT-CERTINFO-SUBJECTPUBLICKEYINFO
	 CRYPT-CERTINFO-CERTIFICATE
	 CRYPT-CERTINFO-USERCERTIFICATE
	 CRYPT-CERTINFO-CACERTIFICATE
	 CRYPT-CERTINFO-ISSUERNAME
	 CRYPT-CERTINFO-VALIDFROM
	 CRYPT-CERTINFO-VALIDTO
	 CRYPT-CERTINFO-SUBJECTNAME
	 CRYPT-CERTINFO-ISSUERUNIQUEID
	 CRYPT-CERTINFO-SUBJECTUNIQUEID
	 CRYPT-CERTINFO-CERTREQUEST
	 CRYPT-CERTINFO-THISUPDATE
	 CRYPT-CERTINFO-NEXTUPDATE
	 CRYPT-CERTINFO-REVOCATIONDATE
	 CRYPT-CERTINFO-REVOCATIONSTATUS
	 CRYPT-CERTINFO-CERTSTATUS
	 CRYPT-CERTINFO-DN
	 CRYPT-CERTINFO-PKIUSER-ID
	 CRYPT-CERTINFO-PKIUSER-ISSUEPASSWORD
	 CRYPT-CERTINFO-PKIUSER-REVPASSWORD
	 CRYPT-CERTINFO-COUNTRYNAME
	 CRYPT-CERTINFO-STATEORPROVINCENAME
	 CRYPT-CERTINFO-LOCALITYNAME
	 CRYPT-CERTINFO-ORGANIZATIONNAME
	 CRYPT-CERTINFO-ORGANISATIONNAME
	 CRYPT-CERTINFO-ORGANIZATIONALUNITNAME
	 CRYPT-CERTINFO-ORGANISATIONALUNITNAME
	 CRYPT-CERTINFO-COMMONNAME
	 CRYPT-CERTINFO-OTHERNAME-TYPEID
	 CRYPT-CERTINFO-OTHERNAME-VALUE
	 CRYPT-CERTINFO-RFC822NAME
	 CRYPT-CERTINFO-EMAIL
	 CRYPT-CERTINFO-DNSNAME
	 CRYPT-CERTINFO-DIRECTORYNAME
	 CRYPT-CERTINFO-EDIPARTYNAME-NAMEASSIGNER
	 CRYPT-CERTINFO-EDIPARTYNAME-PARTYNAME
	 CRYPT-CERTINFO-UNIFORMRESOURCEIDENTIFIER
	 CRYPT-CERTINFO-IPADDRESS
	 CRYPT-CERTINFO-REGISTEREDID
	 CRYPT-CERTINFO-CHALLENGEPASSWORD
	 CRYPT-CERTINFO-CRLEXTREASON
	 CRYPT-CERTINFO-KEYFEATURES
	 CRYPT-CERTINFO-AUTHORITYINFOACCESS
	 CRYPT-CERTINFO-AUTHORITYINFO-RTCS
	 CRYPT-CERTINFO-AUTHORITYINFO-OCSP
	 CRYPT-CERTINFO-AUTHORITYINFO-CAISSUERS
	 CRYPT-CERTINFO-AUTHORITYINFO-CERTSTORE
	 CRYPT-CERTINFO-AUTHORITYINFO-CRLS
	 CRYPT-CERTINFO-OCSP-NONCE
	 CRYPT-CERTINFO-OCSP-RESPONSE
	 CRYPT-CERTINFO-OCSP-RESPONSE-OCSP
	 CRYPT-CERTINFO-OCSP-NOCHECK
	 CRYPT-CERTINFO-OCSP-ARCHIVECUTOFF
	 CRYPT-CERTINFO-SUBJECTINFOACCESS
	 CRYPT-CERTINFO-SUBJECTINFO-CAREPOSITORY
	 CRYPT-CERTINFO-SUBJECTINFO-TIMESTAMPING
	 CRYPT-CERTINFO-SIGG-DATEOFCERTGEN
	 CRYPT-CERTINFO-SIGG-PROCURATION
	 CRYPT-CERTINFO-SIGG-PROCURE-COUNTRY
	 CRYPT-CERTINFO-SIGG-PROCURE-TYPEOFSUBSTITUTION
	 CRYPT-CERTINFO-SIGG-PROCURE-SIGNINGFOR
	 CRYPT-CERTINFO-SIGG-MONETARYLIMIT
	 CRYPT-CERTINFO-SIGG-MONETARY-CURRENCY
	 CRYPT-CERTINFO-SIGG-MONETARY-AMOUNT
	 CRYPT-CERTINFO-SIGG-MONETARY-EXPONENT
	 CRYPT-CERTINFO-SIGG-RESTRICTION
	 CRYPT-CERTINFO-STRONGEXTRANET
	 CRYPT-CERTINFO-STRONGEXTRANET-ZONE
	 CRYPT-CERTINFO-STRONGEXTRANET-ID
	 CRYPT-CERTINFO-SUBJECTDIRECTORYATTRIBUTES
	 CRYPT-CERTINFO-SUBJECTDIR-TYPE
	 CRYPT-CERTINFO-SUBJECTDIR-VALUES
	 CRYPT-CERTINFO-SUBJECTKEYIDENTIFIER
	 CRYPT-CERTINFO-KEYUSAGE
	 CRYPT-CERTINFO-PRIVATEKEYUSAGEPERIOD
	 CRYPT-CERTINFO-PRIVATEKEY-NOTBEFORE
	 CRYPT-CERTINFO-PRIVATEKEY-NOTAFTER
	 CRYPT-CERTINFO-SUBJECTALTNAME
	 CRYPT-CERTINFO-ISSUERALTNAME
	 CRYPT-CERTINFO-BASICCONSTRAINTS
	 CRYPT-CERTINFO-CA
	 CRYPT-CERTINFO-AUTHORITY
	 CRYPT-CERTINFO-PATHLENCONSTRAINT
	 CRYPT-CERTINFO-CRLNUMBER
	 CRYPT-CERTINFO-CRLREASON
	 CRYPT-CERTINFO-HOLDINSTRUCTIONCODE
	 CRYPT-CERTINFO-INVALIDITYDATE
	 CRYPT-CERTINFO-DELTACRLINDICATOR
	 CRYPT-CERTINFO-ISSUINGDISTRIBUTIONPOINT
	 CRYPT-CERTINFO-ISSUINGDIST-FULLNAME
	 CRYPT-CERTINFO-ISSUINGDIST-USERCERTSONLY
	 CRYPT-CERTINFO-ISSUINGDIST-CACERTSONLY
	 CRYPT-CERTINFO-ISSUINGDIST-SOMEREASONSONLY
	 CRYPT-CERTINFO-ISSUINGDIST-INDIRECTCRL
	 CRYPT-CERTINFO-CERTIFICATEISSUER
	 CRYPT-CERTINFO-NAMECONSTRAINTS
	 CRYPT-CERTINFO-PERMITTEDSUBTREES
	 CRYPT-CERTINFO-EXCLUDEDSUBTREES
	 CRYPT-CERTINFO-CRLDISTRIBUTIONPOINT
	 CRYPT-CERTINFO-CRLDIST-FULLNAME
	 CRYPT-CERTINFO-CRLDIST-REASONS
	 CRYPT-CERTINFO-CRLDIST-CRLISSUER
	 CRYPT-CERTINFO-CERTIFICATEPOLICIES
	 CRYPT-CERTINFO-CERTPOLICYID
	 CRYPT-CERTINFO-CERTPOLICY-CPSURI
	 CRYPT-CERTINFO-CERTPOLICY-ORGANIZATION
	 CRYPT-CERTINFO-CERTPOLICY-NOTICENUMBERS
	 CRYPT-CERTINFO-CERTPOLICY-EXPLICITTEXT
	 CRYPT-CERTINFO-POLICYMAPPINGS
	 CRYPT-CERTINFO-ISSUERDOMAINPOLICY
	 CRYPT-CERTINFO-SUBJECTDOMAINPOLICY
	 CRYPT-CERTINFO-AUTHORITYKEYIDENTIFIER
	 CRYPT-CERTINFO-AUTHORITY-KEYIDENTIFIER
	 CRYPT-CERTINFO-AUTHORITY-CERTISSUER
	 CRYPT-CERTINFO-AUTHORITY-CERTSERIALNUMBER
	 CRYPT-CERTINFO-POLICYCONSTRAINTS
	 CRYPT-CERTINFO-REQUIREEXPLICITPOLICY
	 CRYPT-CERTINFO-INHIBITPOLICYMAPPING
	 CRYPT-CERTINFO-EXTKEYUSAGE
	 CRYPT-CERTINFO-EXTKEY-MS-INDIVIDUALCODESIGNING
	 CRYPT-CERTINFO-EXTKEY-MS-COMMERCIALCODESIGNING
	 CRYPT-CERTINFO-EXTKEY-MS-CERTTRUSTLISTSIGNING
	 CRYPT-CERTINFO-EXTKEY-MS-TIMESTAMPSIGNING
	 CRYPT-CERTINFO-EXTKEY-MS-SERVERGATEDCRYPTO
	 CRYPT-CERTINFO-EXTKEY-MS-ENCRYPTEDFILESYSTEM
	 CRYPT-CERTINFO-EXTKEY-SERVERAUTH
	 CRYPT-CERTINFO-EXTKEY-CLIENTAUTH
	 CRYPT-CERTINFO-EXTKEY-CODESIGNING
	 CRYPT-CERTINFO-EXTKEY-EMAILPROTECTION
	 CRYPT-CERTINFO-EXTKEY-IPSECENDSYSTEM
	 CRYPT-CERTINFO-EXTKEY-IPSECTUNNEL
	 CRYPT-CERTINFO-EXTKEY-IPSECUSER
	 CRYPT-CERTINFO-EXTKEY-TIMESTAMPING
	 CRYPT-CERTINFO-EXTKEY-OCSPSIGNING
	 CRYPT-CERTINFO-EXTKEY-DIRECTORYSERVICE
	 CRYPT-CERTINFO-EXTKEY-NS-SERVERGATEDCRYPTO
	 CRYPT-CERTINFO-EXTKEY-VS-SERVERGATEDCRYPTO-CA
	 CRYPT-CERTINFO-FRESHESTCRL
	 CRYPT-CERTINFO-FRESHESTCRL-FULLNAME
	 CRYPT-CERTINFO-FRESHESTCRL-REASONS
	 CRYPT-CERTINFO-FRESHESTCRL-CRLISSUER
	 CRYPT-CERTINFO-INHIBITANYPOLICY
	 CRYPT-CERTINFO-NS-CERTTYPE
	 CRYPT-CERTINFO-NS-BASEURL
	 CRYPT-CERTINFO-NS-REVOCATIONURL
	 CRYPT-CERTINFO-NS-CAREVOCATIONURL
	 CRYPT-CERTINFO-NS-CERTRENEWALURL
	 CRYPT-CERTINFO-NS-CAPOLICYURL
	 CRYPT-CERTINFO-NS-SSLSERVERNAME
	 CRYPT-CERTINFO-NS-COMMENT
	 CRYPT-CERTINFO-SET-HASHEDROOTKEY
	 CRYPT-CERTINFO-SET-ROOTKEYTHUMBPRINT
	 CRYPT-CERTINFO-SET-CERTIFICATETYPE
	 CRYPT-CERTINFO-SET-MERCHANTDATA
	 CRYPT-CERTINFO-SET-MERID
	 CRYPT-CERTINFO-SET-MERACQUIRERBIN
	 CRYPT-CERTINFO-SET-MERCHANTLANGUAGE
	 CRYPT-CERTINFO-SET-MERCHANTNAME
	 CRYPT-CERTINFO-SET-MERCHANTCITY
	 CRYPT-CERTINFO-SET-MERCHANTSTATEPROVINCE
	 CRYPT-CERTINFO-SET-MERCHANTPOSTALCODE
	 CRYPT-CERTINFO-SET-MERCHANTCOUNTRYNAME
	 CRYPT-CERTINFO-SET-MERCOUNTRY
	 CRYPT-CERTINFO-SET-MERAUTHFLAG
	 CRYPT-CERTINFO-SET-CERTCARDREQUIRED
	 CRYPT-CERTINFO-SET-TUNNELING
	 CRYPT-CERTINFO-SET-TUNNELLING
	 CRYPT-CERTINFO-SET-TUNNELINGFLAG
	 CRYPT-CERTINFO-SET-TUNNELLINGFLAG
	 CRYPT-CERTINFO-SET-TUNNELINGALGID
	 CRYPT-CERTINFO-SET-TUNNELLINGALGID
	 CRYPT-CERTINFO-CMS-CONTENTTYPE
	 CRYPT-CERTINFO-CMS-MESSAGEDIGEST
	 CRYPT-CERTINFO-CMS-SIGNINGTIME
	 CRYPT-CERTINFO-CMS-COUNTERSIGNATURE
	 CRYPT-CERTINFO-CMS-SMIMECAPABILITIES
	 CRYPT-CERTINFO-CMS-SMIMECAP-3DES
	 CRYPT-CERTINFO-CMS-SMIMECAP-AES
	 CRYPT-CERTINFO-CMS-SMIMECAP-CAST128
	 CRYPT-CERTINFO-CMS-SMIMECAP-IDEA
	 CRYPT-CERTINFO-CMS-SMIMECAP-RC2
	 CRYPT-CERTINFO-CMS-SMIMECAP-RC5
	 CRYPT-CERTINFO-CMS-SMIMECAP-SKIPJACK
	 CRYPT-CERTINFO-CMS-SMIMECAP-DES
	 CRYPT-CERTINFO-CMS-SMIMECAP-PREFERSIGNEDDATA
	 CRYPT-CERTINFO-CMS-SMIMECAP-CANNOTDECRYPTANY
	 CRYPT-CERTINFO-CMS-RECEIPTREQUEST
	 CRYPT-CERTINFO-CMS-RECEIPT-CONTENTIDENTIFIER
	 CRYPT-CERTINFO-CMS-RECEIPT-FROM
	 CRYPT-CERTINFO-CMS-RECEIPT-TO
	 CRYPT-CERTINFO-CMS-SECURITYLABEL
	 CRYPT-CERTINFO-CMS-SECLABEL-CLASSIFICATION
	 CRYPT-CERTINFO-CMS-SECLABEL-POLICY
	 CRYPT-CERTINFO-CMS-SECLABEL-PRIVACYMARK
	 CRYPT-CERTINFO-CMS-SECLABEL-CATTYPE
	 CRYPT-CERTINFO-CMS-SECLABEL-CATVALUE
	 CRYPT-CERTINFO-CMS-MLEXPANSIONHISTORY
	 CRYPT-CERTINFO-CMS-MLEXP-ENTITYIDENTIFIER
	 CRYPT-CERTINFO-CMS-MLEXP-TIME
	 CRYPT-CERTINFO-CMS-MLEXP-NONE
	 CRYPT-CERTINFO-CMS-MLEXP-INSTEADOF
	 CRYPT-CERTINFO-CMS-MLEXP-INADDITIONTO
	 CRYPT-CERTINFO-CMS-CONTENTHINTS
	 CRYPT-CERTINFO-CMS-CONTENTHINT-DESCRIPTION
	 CRYPT-CERTINFO-CMS-CONTENTHINT-TYPE
	 CRYPT-CERTINFO-CMS-EQUIVALENTLABEL
	 CRYPT-CERTINFO-CMS-EQVLABEL-POLICY
	 CRYPT-CERTINFO-CMS-EQVLABEL-CLASSIFICATION
	 CRYPT-CERTINFO-CMS-EQVLABEL-PRIVACYMARK
	 CRYPT-CERTINFO-CMS-EQVLABEL-CATTYPE
	 CRYPT-CERTINFO-CMS-EQVLABEL-CATVALUE
	 CRYPT-CERTINFO-CMS-SIGNINGCERTIFICATE
	 CRYPT-CERTINFO-CMS-SIGNINGCERT-ESSCERTID
	 CRYPT-CERTINFO-CMS-SIGNINGCERT-POLICIES
	 CRYPT-CERTINFO-CMS-SIGTYPEIDENTIFIER
	 CRYPT-CERTINFO-CMS-SIGTYPEID-ORIGINATORSIG
	 CRYPT-CERTINFO-CMS-SIGTYPEID-DOMAINSIG
	 CRYPT-CERTINFO-CMS-SIGTYPEID-ADDITIONALATTRIBUTES
	 CRYPT-CERTINFO-CMS-SIGTYPEID-REVIEWSIG
	 CRYPT-CERTINFO-CMS-NONCE
	 CRYPT-CERTINFO-SCEP-MESSAGETYPE
	 CRYPT-CERTINFO-SCEP-PKISTATUS
	 CRYPT-CERTINFO-SCEP-FAILINFO
	 CRYPT-CERTINFO-SCEP-SENDERNONCE
	 CRYPT-CERTINFO-SCEP-RECIPIENTNONCE
	 CRYPT-CERTINFO-SCEP-TRANSACTIONID
	 CRYPT-CERTINFO-CMS-SPCAGENCYINFO
	 CRYPT-CERTINFO-CMS-SPCAGENCYURL
	 CRYPT-CERTINFO-CMS-SPCSTATEMENTTYPE
	 CRYPT-CERTINFO-CMS-SPCSTMT-INDIVIDUALCODESIGNING
	 CRYPT-CERTINFO-CMS-SPCSTMT-COMMERCIALCODESIGNING
	 CRYPT-CERTINFO-CMS-SPCOPUSINFO
	 CRYPT-CERTINFO-CMS-SPCOPUSINFO-NAME
	 CRYPT-CERTINFO-CMS-SPCOPUSINFO-URL
	 CRYPT-KEYINFO-QUERY
	 CRYPT-KEYINFO-QUERY-REQUESTS
	 CRYPT-DEVINFO-INITIALISE
	 CRYPT-DEVINFO-INITIALIZE
	 CRYPT-DEVINFO-AUTHENT-USER
	 CRYPT-DEVINFO-AUTHENT-SUPERVISOR
	 CRYPT-DEVINFO-SET-AUTHENT-USER
	 CRYPT-DEVINFO-SET-AUTHENT-SUPERVISOR
	 CRYPT-DEVINFO-ZEROISE
	 CRYPT-DEVINFO-ZEROIZE
	 CRYPT-DEVINFO-LOGGEDIN
	 CRYPT-DEVINFO-LABEL
	 CRYPT-ENVINFO-DATASIZE
	 CRYPT-ENVINFO-COMPRESSION
	 CRYPT-ENVINFO-CONTENTTYPE
	 CRYPT-ENVINFO-DETACHEDSIGNATURE
	 CRYPT-ENVINFO-SIGNATURE-RESULT
	 CRYPT-ENVINFO-MAC
	 CRYPT-ENVINFO-PASSWORD
	 CRYPT-ENVINFO-KEY
	 CRYPT-ENVINFO-SIGNATURE
	 CRYPT-ENVINFO-SIGNATURE-EXTRADATA
	 CRYPT-ENVINFO-RECIPIENT
	 CRYPT-ENVINFO-PUBLICKEY
	 CRYPT-ENVINFO-PRIVATEKEY
	 CRYPT-ENVINFO-PRIVATEKEY-LABEL
	 CRYPT-ENVINFO-ORIGINATOR
	 CRYPT-ENVINFO-SESSIONKEY
	 CRYPT-ENVINFO-HASH
	 CRYPT-ENVINFO-TIMESTAMP
	 CRYPT-ENVINFO-KEYSET-SIGCHECK
	 CRYPT-ENVINFO-KEYSET-ENCRYPT
	 CRYPT-ENVINFO-KEYSET-DECRYPT
	 CRYPT-SESSINFO-ACTIVE
	 CRYPT-SESSINFO-CONNECTIONACTIVE
	 CRYPT-SESSINFO-USERNAME
	 CRYPT-SESSINFO-PASSWORD
	 CRYPT-SESSINFO-PRIVATEKEY
	 CRYPT-SESSINFO-KEYSET
	 CRYPT-SESSINFO-AUTHRESPONSE
	 CRYPT-SESSINFO-SERVER-NAME
	 CRYPT-SESSINFO-SERVER-PORT
	 CRYPT-SESSINFO-SERVER-FINGERPRINT
	 CRYPT-SESSINFO-CLIENT-NAME
	 CRYPT-SESSINFO-CLIENT-PORT
	 CRYPT-SESSINFO-SESSION
	 CRYPT-SESSINFO-NETWORKSOCKET
	 CRYPT-SESSINFO-VERSION
	 CRYPT-SESSINFO-REQUEST
	 CRYPT-SESSINFO-RESPONSE
	 CRYPT-SESSINFO-CACERTIFICATE
	 CRYPT-SESSINFO-TSP-MSGIMPRINT
	 CRYPT-SESSINFO-CMP-REQUESTTYPE
	 CRYPT-SESSINFO-CMP-PKIBOOT
	 CRYPT-SESSINFO-CMP-PRIVKEYSET
	 CRYPT-SESSINFO-SSH-CHANNEL
	 CRYPT-SESSINFO-SSH-CHANNEL-TYPE
	 CRYPT-SESSINFO-SSH-CHANNEL-ARG1
	 CRYPT-SESSINFO-SSH-CHANNEL-ARG2
	 CRYPT-SESSINFO-SSH-CHANNEL-ACTIVE
	 CRYPT-USERINFO-PASSWORD
	 CRYPT-USERINFO-CAKEY-CERTSIGN
	 CRYPT-USERINFO-CAKEY-CRLSIGN
	 CRYPT-USERINFO-CAKEY-RTCSSIGN
	 CRYPT-USERINFO-CAKEY-OCSPSIGN
	 CRYPT-KEYUSAGE-NONE
	 CRYPT-KEYUSAGE-DIGITALSIGNATURE
	 CRYPT-KEYUSAGE-NONREPUDIATION
	 CRYPT-KEYUSAGE-KEYENCIPHERMENT
	 CRYPT-KEYUSAGE-DATAENCIPHERMENT
	 CRYPT-KEYUSAGE-KEYAGREEMENT
	 CRYPT-KEYUSAGE-KEYCERTSIGN
	 CRYPT-KEYUSAGE-CRLSIGN
	 CRYPT-KEYUSAGE-ENCIPHERONLY
	 CRYPT-KEYUSAGE-DECIPHERONLY
	 CRYPT-KEYUSAGE-LAST
	 CRYPT-CRLREASON-UNSPECIFIED
	 CRYPT-CRLREASON-KEYCOMPROMISE
	 CRYPT-CRLREASON-CACOMPROMISE
	 CRYPT-CRLREASON-AFFILIATIONCHANGED
	 CRYPT-CRLREASON-SUPERSEDED
	 CRYPT-CRLREASON-CESSATIONOFOPERATION
	 CRYPT-CRLREASON-CERTIFICATEHOLD
	 CRYPT-CRLREASON-REMOVEFROMCRL
	 CRYPT-CRLREASON-LAST
	 CRYPT-CRLREASON-NEVERVALID
	 CRYPT-CRLEXTREASON-LAST
	 CRYPT-CRLREASONFLAG-UNUSED
	 CRYPT-CRLREASONFLAG-KEYCOMPROMISE
	 CRYPT-CRLREASONFLAG-CACOMPROMISE
	 CRYPT-CRLREASONFLAG-AFFILIATIONCHANGED
	 CRYPT-CRLREASONFLAG-SUPERSEDED
	 CRYPT-CRLREASONFLAG-CESSATIONOFOPERATION
	 CRYPT-CRLREASONFLAG-CERTIFICATEHOLD
	 CRYPT-CRLREASONFLAG-LAST
	 CRYPT-HOLDINSTRUCTION-NONE
	 CRYPT-HOLDINSTRUCTION-CALLISSUER
	 CRYPT-HOLDINSTRUCTION-REJECT
	 CRYPT-HOLDINSTRUCTION-PICKUPTOKEN
	 CRYPT-HOLDINSTRUCTION-LAST
	 CRYPT-COMPLIANCELEVEL-OBLIVIOUS
	 CRYPT-COMPLIANCELEVEL-REDUCED
	 CRYPT-COMPLIANCELEVEL-STANDARD
	 CRYPT-COMPLIANCELEVEL-PKIX-PARTIAL
	 CRYPT-COMPLIANCELEVEL-PKIX-FULL
	 CRYPT-COMPLIANCELEVEL-LAST
	 CRYPT-NS-CERTTYPE-SSLCLIENT
	 CRYPT-NS-CERTTYPE-SSLSERVER
	 CRYPT-NS-CERTTYPE-SMIME
	 CRYPT-NS-CERTTYPE-OBJECTSIGNING
	 CRYPT-NS-CERTTYPE-RESERVED
	 CRYPT-NS-CERTTYPE-SSLCA
	 CRYPT-NS-CERTTYPE-SMIMECA
	 CRYPT-NS-CERTTYPE-OBJECTSIGNINGCA
	 CRYPT-NS-CERTTYPE-LAST
	 CRYPT-SET-CERTTYPE-CARD
	 CRYPT-SET-CERTTYPE-MER
	 CRYPT-SET-CERTTYPE-PGWY
	 CRYPT-SET-CERTTYPE-CCA
	 CRYPT-SET-CERTTYPE-MCA
	 CRYPT-SET-CERTTYPE-PCA
	 CRYPT-SET-CERTTYPE-GCA
	 CRYPT-SET-CERTTYPE-BCA
	 CRYPT-SET-CERTTYPE-RCA
	 CRYPT-SET-CERTTYPE-ACQ
	 CRYPT-SET-CERTTYPE-LAST
	 CRYPT-CONTENT-NONE
	 CRYPT-CONTENT-DATA
	 CRYPT-CONTENT-SIGNEDDATA
	 CRYPT-CONTENT-ENVELOPEDDATA
	 CRYPT-CONTENT-SIGNEDANDENVELOPEDDATA
	 CRYPT-CONTENT-DIGESTEDDATA
	 CRYPT-CONTENT-ENCRYPTEDDATA
	 CRYPT-CONTENT-COMPRESSEDDATA
	 CRYPT-CONTENT-TSTINFO
	 CRYPT-CONTENT-SPCINDIRECTDATACONTEXT
	 CRYPT-CONTENT-RTCSREQUEST
	 CRYPT-CONTENT-RTCSRESPONSE
	 CRYPT-CONTENT-RTCSRESPONSE-EXT
	 CRYPT-CONTENT-LAST
	 CRYPT-CLASSIFICATION-UNMARKED
	 CRYPT-CLASSIFICATION-UNCLASSIFIED
	 CRYPT-CLASSIFICATION-RESTRICTED
	 CRYPT-CLASSIFICATION-CONFIDENTIAL
	 CRYPT-CLASSIFICATION-SECRET
	 CRYPT-CLASSIFICATION-TOP-SECRET
	 CRYPT-CLASSIFICATION-LAST
	 CRYPT-CERTSTATUS-VALID
	 CRYPT-CERTSTATUS-NOTVALID
	 CRYPT-CERTSTATUS-NONAUTHORITATIVE
	 CRYPT-CERTSTATUS-UNKNOWN
	 CRYPT-OCSPSTATUS-NOTREVOKED
	 CRYPT-OCSPSTATUS-REVOKED
	 CRYPT-OCSPSTATUS-UNKNOWN
	 CRYPT-SIGNATURELEVEL-NONE
	 CRYPT-SIGNATURELEVEL-SIGNERCERT
	 CRYPT-SIGNATURELEVEL-ALL
	 CRYPT-SIGNATURELEVEL-LAST
	 CRYPT-CERTFORMAT-NONE
	 CRYPT-CERTFORMAT-CERTIFICATE
	 CRYPT-CERTFORMAT-CERTCHAIN
	 CRYPT-CERTFORMAT-TEXT-CERTIFICATE
	 CRYPT-CERTFORMAT-TEXT-CERTCHAIN
	 CRYPT-CERTFORMAT-XML-CERTIFICATE
	 CRYPT-CERTFORMAT-XML-CERTCHAIN
	 CRYPT-CERTFORMAT-LAST
	 CRYPT-REQUESTTYPE-NONE
	 CRYPT-REQUESTTYPE-INITIALISATION
	 CRYPT-REQUESTTYPE-INITIALIZATION
	 CRYPT-REQUESTTYPE-CERTIFICATE
	 CRYPT-REQUESTTYPE-KEYUPDATE
	 CRYPT-REQUESTTYPE-REVOCATION
	 CRYPT-REQUESTTYPE-PKIBOOT
	 CRYPT-REQUESTTYPE-LAST
	 CRYPT-KEYID-NONE
	 CRYPT-KEYID-NAME
	 CRYPT-KEYID-EMAIL
	 CRYPT-KEYID-LAST
	 CRYPT-OBJECT-NONE
	 CRYPT-OBJECT-ENCRYPTED-KEY
	 CRYPT-OBJECT-PKCENCRYPTED-KEY
	 CRYPT-OBJECT-KEYAGREEMENT
	 CRYPT-OBJECT-SIGNATURE
	 CRYPT-OBJECT-LAST
	 CRYPT-ERRTYPE-NONE
	 CRYPT-ERRTYPE-ATTR-SIZE
	 CRYPT-ERRTYPE-ATTR-VALUE
	 CRYPT-ERRTYPE-ATTR-ABSENT
	 CRYPT-ERRTYPE-ATTR-PRESENT
	 CRYPT-ERRTYPE-CONSTRAINT
	 CRYPT-ERRTYPE-ISSUERCONSTRAINT
	 CRYPT-ERRTYPE-LAST
	 CRYPT-CERTACTION-NONE
	 CRYPT-CERTACTION-CREATE
	 CRYPT-CERTACTION-CONNECT
	 CRYPT-CERTACTION-DISCONNECT
	 CRYPT-CERTACTION-ERROR
	 CRYPT-CERTACTION-ADDUSER
	 CRYPT-CERTACTION-REQUEST-CERT
	 CRYPT-CERTACTION-REQUEST-RENEWAL
	 CRYPT-CERTACTION-REQUEST-REVOCATION
	 CRYPT-CERTACTION-CERT-CREATION
	 CRYPT-CERTACTION-CERT-CREATION-COMPLETE
	 CRYPT-CERTACTION-CERT-CREATION-DROP
	 CRYPT-CERTACTION-CERT-CREATION-REVERSE
	 CRYPT-CERTACTION-RESTART-CLEANUP
	 CRYPT-CERTACTION-RESTART-REVOKE-CERT
	 CRYPT-CERTACTION-ISSUE-CERT
	 CRYPT-CERTACTION-ISSUE-CRL
	 CRYPT-CERTACTION-REVOKE-CERT
	 CRYPT-CERTACTION-EXPIRE-CERT
	 CRYPT-CERTACTION-CLEANUP
	 CRYPT-CERTACTION-LAST
	 CRYPT-MAX-KEYSIZE
	 CRYPT-MAX-IVSIZE
	 CRYPT-MAX-PKCSIZE
	 CRYPT-MAX-HASHSIZE
	 CRYPT-MAX-TEXTSIZE
	 CRYPT-USE-DEFAULT
	 CRYPT-UNUSED
	 CRYPT-KEYTYPE-PRIVATE
	 CRYPT-KEYTYPE-PUBLIC
	 CRYPT-RANDOM-FASTPOLL
	 CRYPT-RANDOM-SLOWPOLL
	 CRYPT-CURSOR-FIRST
	 CRYPT-CURSOR-PREVIOUS
	 CRYPT-CURSOR-NEXT
	 CRYPT-CURSOR-LAST
	 CRYPT-KEYOPT-NONE
	 CRYPT-KEYOPT-READONLY
	 CRYPT-KEYOPT-CREATE
	 CRYPT-KEYOPT-LAST
	 CRYPT-OK
	 CRYPT-ERROR-PARAM1
	 CRYPT-ERROR-PARAM2
	 CRYPT-ERROR-PARAM3
	 CRYPT-ERROR-PARAM4
	 CRYPT-ERROR-PARAM5
	 CRYPT-ERROR-PARAM6
	 CRYPT-ERROR-PARAM7
	 CRYPT-ERROR-MEMORY
	 CRYPT-ERROR-NOTINITED
	 CRYPT-ERROR-INITED
	 CRYPT-ERROR-NOSECURE
	 CRYPT-ERROR-RANDOM
	 CRYPT-ERROR-FAILED
	 CRYPT-ERROR-NOTAVAIL
	 CRYPT-ERROR-PERMISSION
	 CRYPT-ERROR-WRONGKEY
	 CRYPT-ERROR-INCOMPLETE
	 CRYPT-ERROR-COMPLETE
	 CRYPT-ERROR-TIMEOUT
	 CRYPT-ERROR-INVALID
	 CRYPT-ERROR-SIGNALLED
	 CRYPT-ERROR-OVERFLOW
	 CRYPT-ERROR-UNDERFLOW
	 CRYPT-ERROR-BADDATA
	 CRYPT-ERROR-SIGNATURE
	 CRYPT-ERROR-OPEN
	 CRYPT-ERROR-READ
	 CRYPT-ERROR-WRITE
	 CRYPT-ERROR-NOTFOUND
	 CRYPT-ERROR-DUPLICATE
	 CRYPT-ENVELOPE-RESOURCE
	 crypt-add-cert-extension
	 crypt-add-private-key
	 crypt-add-public-key
	 crypt-add-random
	 crypt-async-cancel
	 crypt-async-query
	 crypt-ca-add-item
	 crypt-ca-cert-management
	 crypt-ca-get-item
	 crypt-check-cert
	 crypt-check-signature
	 crypt-check-signature-ex
	 crypt-create-cert
	 crypt-create-context
	 crypt-create-envelope
	 crypt-create-session
	 crypt-create-signature
	 crypt-create-signature-ex
	 crypt-decrypt
	 crypt-delete-attribute
	 crypt-delete-cert-extension
	 crypt-delete-key
	 crypt-destroy-cert
	 crypt-destroy-context
	 crypt-destroy-envelope
	 crypt-destroy-object
	 crypt-destroy-session
	 crypt-device-close
	 crypt-device-create-context
	 crypt-device-open
	 crypt-device-query-capability
	 crypt-encrypt
	 crypt-end
	 crypt-export-cert
	 crypt-export-key
	 crypt-export-key-ex
	 crypt-flush-data
	 crypt-generate-key
	 crypt-generate-key-async
	 crypt-get-attribute
	 crypt-get-attribute-string
	 crypt-get-cert-extension
	 crypt-get-private-key
	 crypt-get-public-key
	 crypt-import-cert
	 crypt-import-key
	 crypt-init
	 crypt-keyset-close
	 crypt-keyset-open
	 crypt-pop-data
	 crypt-push-data
	 crypt-query-capability
	 crypt-query-object
	 crypt-set-attribute
	 crypt-set-attribute-string
	 crypt-sign-cert
	 crypt-input-port
	 crypt-output-port))

#>
#include <cryptlib.h>
<#

(define CRYPT-ALGO-NONE (foreign-value "CRYPT_ALGO_NONE" int))
(define CRYPT-ALGO-DES (foreign-value "CRYPT_ALGO_DES" int))
(define CRYPT-ALGO-3DES (foreign-value "CRYPT_ALGO_3DES" int))
(define CRYPT-ALGO-IDEA (foreign-value "CRYPT_ALGO_IDEA" int))
(define CRYPT-ALGO-CAST (foreign-value "CRYPT_ALGO_CAST" int))
(define CRYPT-ALGO-RC2 (foreign-value "CRYPT_ALGO_RC2" int))
(define CRYPT-ALGO-RC4 (foreign-value "CRYPT_ALGO_RC4" int))
(define CRYPT-ALGO-RC5 (foreign-value "CRYPT_ALGO_RC5" int))
(define CRYPT-ALGO-AES (foreign-value "CRYPT_ALGO_AES" int))
(define CRYPT-ALGO-BLOWFISH (foreign-value "CRYPT_ALGO_BLOWFISH" int))
(define CRYPT-ALGO-SKIPJACK (foreign-value "CRYPT_ALGO_SKIPJACK" int))
(define CRYPT-ALGO-DH (foreign-value "CRYPT_ALGO_DH" int))
(define CRYPT-ALGO-RSA (foreign-value "CRYPT_ALGO_RSA" int))
(define CRYPT-ALGO-DSA (foreign-value "CRYPT_ALGO_DSA" int))
(define CRYPT-ALGO-ELGAMAL (foreign-value "CRYPT_ALGO_ELGAMAL" int))
(define CRYPT-ALGO-KEA (foreign-value "CRYPT_ALGO_KEA" int))
(define CRYPT-ALGO-MD2 (foreign-value "CRYPT_ALGO_MD2" int))
(define CRYPT-ALGO-MD4 (foreign-value "CRYPT_ALGO_MD4" int))
(define CRYPT-ALGO-MD5 (foreign-value "CRYPT_ALGO_MD5" int))
(define CRYPT-ALGO-SHA (foreign-value "CRYPT_ALGO_SHA" int))
(define CRYPT-ALGO-RIPEMD160 (foreign-value "CRYPT_ALGO_RIPEMD160" int))
(define CRYPT-ALGO-HMAC-MD5 (foreign-value "CRYPT_ALGO_HMAC_MD5" int))
(define CRYPT-ALGO-HMAC-SHA (foreign-value "CRYPT_ALGO_HMAC_SHA" int))
(define CRYPT-ALGO-HMAC-RIPEMD160 (foreign-value "CRYPT_ALGO_HMAC_RIPEMD160" int))
(define CRYPT-ALGO-LAST (foreign-value "CRYPT_ALGO_LAST" int))
(define CRYPT-ALGO-FIRST-CONVENTIONAL (foreign-value "CRYPT_ALGO_FIRST_CONVENTIONAL" int))
(define CRYPT-ALGO-LAST-CONVENTIONAL (foreign-value "CRYPT_ALGO_LAST_CONVENTIONAL" int))
(define CRYPT-ALGO-FIRST-PKC (foreign-value "CRYPT_ALGO_FIRST_PKC" int))
(define CRYPT-ALGO-LAST-PKC (foreign-value "CRYPT_ALGO_LAST_PKC" int))
(define CRYPT-ALGO-FIRST-HASH (foreign-value "CRYPT_ALGO_FIRST_HASH" int))
(define CRYPT-ALGO-LAST-HASH (foreign-value "CRYPT_ALGO_LAST_HASH" int))
(define CRYPT-ALGO-FIRST-MAC (foreign-value "CRYPT_ALGO_FIRST_MAC" int))
(define CRYPT-ALGO-LAST-MAC (foreign-value "CRYPT_ALGO_LAST_MAC" int))
(define CRYPT-MODE-NONE (foreign-value "CRYPT_MODE_NONE" int))
(define CRYPT-MODE-ECB (foreign-value "CRYPT_MODE_ECB" int))
(define CRYPT-MODE-CBC (foreign-value "CRYPT_MODE_CBC" int))
(define CRYPT-MODE-CFB (foreign-value "CRYPT_MODE_CFB" int))
(define CRYPT-MODE-OFB (foreign-value "CRYPT_MODE_OFB" int))
(define CRYPT-MODE-LAST (foreign-value "CRYPT_MODE_LAST" int))
(define CRYPT-KEYSET-NONE (foreign-value "CRYPT_KEYSET_NONE" int))
(define CRYPT-KEYSET-FILE (foreign-value "CRYPT_KEYSET_FILE" int))
(define CRYPT-KEYSET-HTTP (foreign-value "CRYPT_KEYSET_HTTP" int))
(define CRYPT-KEYSET-LDAP (foreign-value "CRYPT_KEYSET_LDAP" int))
(define CRYPT-KEYSET-ODBC (foreign-value "CRYPT_KEYSET_ODBC" int))
(define CRYPT-KEYSET-DATABASE (foreign-value "CRYPT_KEYSET_DATABASE" int))
(define CRYPT-KEYSET-PLUGIN (foreign-value "CRYPT_KEYSET_PLUGIN" int))
(define CRYPT-KEYSET-ODBC-STORE (foreign-value "CRYPT_KEYSET_ODBC_STORE" int))
(define CRYPT-KEYSET-DATABASE-STORE (foreign-value "CRYPT_KEYSET_DATABASE_STORE" int))
(define CRYPT-KEYSET-PLUGIN-STORE (foreign-value "CRYPT_KEYSET_PLUGIN_STORE" int))
(define CRYPT-KEYSET-LAST (foreign-value "CRYPT_KEYSET_LAST" int))
(define CRYPT-DEVICE-NONE (foreign-value "CRYPT_DEVICE_NONE" int))
(define CRYPT-DEVICE-FORTEZZA (foreign-value "CRYPT_DEVICE_FORTEZZA" int))
(define CRYPT-DEVICE-PKCS11 (foreign-value "CRYPT_DEVICE_PKCS11" int))
(define CRYPT-DEVICE-CRYPTOAPI (foreign-value "CRYPT_DEVICE_CRYPTOAPI" int))
(define CRYPT-DEVICE-LAST (foreign-value "CRYPT_DEVICE_LAST" int))
(define CRYPT-CERTTYPE-NONE (foreign-value "CRYPT_CERTTYPE_NONE" int))
(define CRYPT-CERTTYPE-CERTIFICATE (foreign-value "CRYPT_CERTTYPE_CERTIFICATE" int))
(define CRYPT-CERTTYPE-ATTRIBUTE-CERT (foreign-value "CRYPT_CERTTYPE_ATTRIBUTE_CERT" int))
(define CRYPT-CERTTYPE-CERTCHAIN (foreign-value "CRYPT_CERTTYPE_CERTCHAIN" int))
(define CRYPT-CERTTYPE-CERTREQUEST (foreign-value "CRYPT_CERTTYPE_CERTREQUEST" int))
(define CRYPT-CERTTYPE-REQUEST-CERT (foreign-value "CRYPT_CERTTYPE_REQUEST_CERT" int))
(define CRYPT-CERTTYPE-REQUEST-REVOCATION (foreign-value "CRYPT_CERTTYPE_REQUEST_REVOCATION" int))
(define CRYPT-CERTTYPE-CRL (foreign-value "CRYPT_CERTTYPE_CRL" int))
(define CRYPT-CERTTYPE-CMS-ATTRIBUTES (foreign-value "CRYPT_CERTTYPE_CMS_ATTRIBUTES" int))
(define CRYPT-CERTTYPE-RTCS-REQUEST (foreign-value "CRYPT_CERTTYPE_RTCS_REQUEST" int))
(define CRYPT-CERTTYPE-RTCS-RESPONSE (foreign-value "CRYPT_CERTTYPE_RTCS_RESPONSE" int))
(define CRYPT-CERTTYPE-OCSP-REQUEST (foreign-value "CRYPT_CERTTYPE_OCSP_REQUEST" int))
(define CRYPT-CERTTYPE-OCSP-RESPONSE (foreign-value "CRYPT_CERTTYPE_OCSP_RESPONSE" int))
(define CRYPT-CERTTYPE-PKIUSER (foreign-value "CRYPT_CERTTYPE_PKIUSER" int))
(define CRYPT-CERTTYPE-LAST (foreign-value "CRYPT_CERTTYPE_LAST" int))
(define CRYPT-FORMAT-NONE (foreign-value "CRYPT_FORMAT_NONE" int))
(define CRYPT-FORMAT-AUTO (foreign-value "CRYPT_FORMAT_AUTO" int))
(define CRYPT-FORMAT-CRYPTLIB (foreign-value "CRYPT_FORMAT_CRYPTLIB" int))
(define CRYPT-FORMAT-CMS (foreign-value "CRYPT_FORMAT_CMS" int))
(define CRYPT-FORMAT-PKCS7 (foreign-value "CRYPT_FORMAT_PKCS7" int))
(define CRYPT-FORMAT-SMIME (foreign-value "CRYPT_FORMAT_SMIME" int))
(define CRYPT-FORMAT-PGP (foreign-value "CRYPT_FORMAT_PGP" int))
(define CRYPT-FORMAT-LAST (foreign-value "CRYPT_FORMAT_LAST" int))
(define CRYPT-SESSION-NONE (foreign-value "CRYPT_SESSION_NONE" int))
(define CRYPT-SESSION-SSH (foreign-value "CRYPT_SESSION_SSH" int))
(define CRYPT-SESSION-SSH-SERVER (foreign-value "CRYPT_SESSION_SSH_SERVER" int))
(define CRYPT-SESSION-SSL (foreign-value "CRYPT_SESSION_SSL" int))
(define CRYPT-SESSION-SSL-SERVER (foreign-value "CRYPT_SESSION_SSL_SERVER" int))
(define CRYPT-SESSION-RTCS (foreign-value "CRYPT_SESSION_RTCS" int))
(define CRYPT-SESSION-RTCS-SERVER (foreign-value "CRYPT_SESSION_RTCS_SERVER" int))
(define CRYPT-SESSION-OCSP (foreign-value "CRYPT_SESSION_OCSP" int))
(define CRYPT-SESSION-OCSP-SERVER (foreign-value "CRYPT_SESSION_OCSP_SERVER" int))
(define CRYPT-SESSION-TSP (foreign-value "CRYPT_SESSION_TSP" int))
(define CRYPT-SESSION-TSP-SERVER (foreign-value "CRYPT_SESSION_TSP_SERVER" int))
(define CRYPT-SESSION-CMP (foreign-value "CRYPT_SESSION_CMP" int))
(define CRYPT-SESSION-CMP-SERVER (foreign-value "CRYPT_SESSION_CMP_SERVER" int))
(define CRYPT-SESSION-SCEP (foreign-value "CRYPT_SESSION_SCEP" int))
(define CRYPT-SESSION-SCEP-SERVER (foreign-value "CRYPT_SESSION_SCEP_SERVER" int))
(define CRYPT-SESSION-LAST (foreign-value "CRYPT_SESSION_LAST" int))
(define CRYPT-USER-NONE (foreign-value "CRYPT_USER_NONE" int))
(define CRYPT-USER-NORMAL (foreign-value "CRYPT_USER_NORMAL" int))
(define CRYPT-USER-SO (foreign-value "CRYPT_USER_SO" int))
(define CRYPT-USER-CA (foreign-value "CRYPT_USER_CA" int))
(define CRYPT-USER-LAST (foreign-value "CRYPT_USER_LAST" int))
(define CRYPT-ATTRIBUTE-NONE (foreign-value "CRYPT_ATTRIBUTE_NONE" int))
(define CRYPT-PROPERTY-HIGHSECURITY (foreign-value "CRYPT_PROPERTY_HIGHSECURITY" int))
(define CRYPT-PROPERTY-OWNER (foreign-value "CRYPT_PROPERTY_OWNER" int))
(define CRYPT-PROPERTY-FORWARDCOUNT (foreign-value "CRYPT_PROPERTY_FORWARDCOUNT" int))
(define CRYPT-PROPERTY-LOCKED (foreign-value "CRYPT_PROPERTY_LOCKED" int))
(define CRYPT-PROPERTY-USAGECOUNT (foreign-value "CRYPT_PROPERTY_USAGECOUNT" int))
(define CRYPT-PROPERTY-NONEXPORTABLE (foreign-value "CRYPT_PROPERTY_NONEXPORTABLE" int))
(define CRYPT-ATTRIBUTE-ERRORTYPE (foreign-value "CRYPT_ATTRIBUTE_ERRORTYPE" int))
(define CRYPT-ATTRIBUTE-ERRORLOCUS (foreign-value "CRYPT_ATTRIBUTE_ERRORLOCUS" int))
(define CRYPT-ATTRIBUTE-INT-ERRORCODE (foreign-value "CRYPT_ATTRIBUTE_INT_ERRORCODE" int))
(define CRYPT-ATTRIBUTE-INT-ERRORMESSAGE (foreign-value "CRYPT_ATTRIBUTE_INT_ERRORMESSAGE" int))
(define CRYPT-ATTRIBUTE-BUFFERSIZE (foreign-value "CRYPT_ATTRIBUTE_BUFFERSIZE" int))
(define CRYPT-OPTION-INFO-DESCRIPTION (foreign-value "CRYPT_OPTION_INFO_DESCRIPTION" int))
(define CRYPT-OPTION-INFO-COPYRIGHT (foreign-value "CRYPT_OPTION_INFO_COPYRIGHT" int))
(define CRYPT-OPTION-INFO-MAJORVERSION (foreign-value "CRYPT_OPTION_INFO_MAJORVERSION" int))
(define CRYPT-OPTION-INFO-MINORVERSION (foreign-value "CRYPT_OPTION_INFO_MINORVERSION" int))
(define CRYPT-OPTION-INFO-STEPPING (foreign-value "CRYPT_OPTION_INFO_STEPPING" int))
(define CRYPT-OPTION-ENCR-ALGO (foreign-value "CRYPT_OPTION_ENCR_ALGO" int))
(define CRYPT-OPTION-ENCR-HASH (foreign-value "CRYPT_OPTION_ENCR_HASH" int))
(define CRYPT-OPTION-ENCR-MAC (foreign-value "CRYPT_OPTION_ENCR_MAC" int))
(define CRYPT-OPTION-PKC-ALGO (foreign-value "CRYPT_OPTION_PKC_ALGO" int))
(define CRYPT-OPTION-PKC-KEYSIZE (foreign-value "CRYPT_OPTION_PKC_KEYSIZE" int))
(define CRYPT-OPTION-SIG-ALGO (foreign-value "CRYPT_OPTION_SIG_ALGO" int))
(define CRYPT-OPTION-SIG-KEYSIZE (foreign-value "CRYPT_OPTION_SIG_KEYSIZE" int))
(define CRYPT-OPTION-KEYING-ALGO (foreign-value "CRYPT_OPTION_KEYING_ALGO" int))
(define CRYPT-OPTION-KEYING-ITERATIONS (foreign-value "CRYPT_OPTION_KEYING_ITERATIONS" int))
(define CRYPT-OPTION-CERT-SIGNUNRECOGNISEDATTRIBUTES (foreign-value "CRYPT_OPTION_CERT_SIGNUNRECOGNISEDATTRIBUTES" int))
(define CRYPT-OPTION-CERT-VALIDITY (foreign-value "CRYPT_OPTION_CERT_VALIDITY" int))
(define CRYPT-OPTION-CERT-UPDATEINTERVAL (foreign-value "CRYPT_OPTION_CERT_UPDATEINTERVAL" int))
(define CRYPT-OPTION-CERT-COMPLIANCELEVEL (foreign-value "CRYPT_OPTION_CERT_COMPLIANCELEVEL" int))
(define CRYPT-OPTION-CMS-DEFAULTATTRIBUTES (foreign-value "CRYPT_OPTION_CMS_DEFAULTATTRIBUTES" int))
(define CRYPT-OPTION-SMIME-DEFAULTATTRIBUTES (foreign-value "CRYPT_OPTION_SMIME_DEFAULTATTRIBUTES" int))
(define CRYPT-OPTION-KEYS-LDAP-OBJECTCLASS (foreign-value "CRYPT_OPTION_KEYS_LDAP_OBJECTCLASS" int))
(define CRYPT-OPTION-KEYS-LDAP-OBJECTTYPE (foreign-value "CRYPT_OPTION_KEYS_LDAP_OBJECTTYPE" int))
(define CRYPT-OPTION-KEYS-LDAP-FILTER (foreign-value "CRYPT_OPTION_KEYS_LDAP_FILTER" int))
(define CRYPT-OPTION-KEYS-LDAP-CACERTNAME (foreign-value "CRYPT_OPTION_KEYS_LDAP_CACERTNAME" int))
(define CRYPT-OPTION-KEYS-LDAP-CERTNAME (foreign-value "CRYPT_OPTION_KEYS_LDAP_CERTNAME" int))
(define CRYPT-OPTION-KEYS-LDAP-CRLNAME (foreign-value "CRYPT_OPTION_KEYS_LDAP_CRLNAME" int))
(define CRYPT-OPTION-KEYS-LDAP-EMAILNAME (foreign-value "CRYPT_OPTION_KEYS_LDAP_EMAILNAME" int))
(define CRYPT-OPTION-DEVICE-PKCS11-DVR01 (foreign-value "CRYPT_OPTION_DEVICE_PKCS11_DVR01" int))
(define CRYPT-OPTION-DEVICE-PKCS11-DVR02 (foreign-value "CRYPT_OPTION_DEVICE_PKCS11_DVR02" int))
(define CRYPT-OPTION-DEVICE-PKCS11-DVR03 (foreign-value "CRYPT_OPTION_DEVICE_PKCS11_DVR03" int))
(define CRYPT-OPTION-DEVICE-PKCS11-DVR04 (foreign-value "CRYPT_OPTION_DEVICE_PKCS11_DVR04" int))
(define CRYPT-OPTION-DEVICE-PKCS11-DVR05 (foreign-value "CRYPT_OPTION_DEVICE_PKCS11_DVR05" int))
(define CRYPT-OPTION-DEVICE-PKCS11-HARDWAREONLY (foreign-value "CRYPT_OPTION_DEVICE_PKCS11_HARDWAREONLY" int))
(define CRYPT-OPTION-NET-SOCKS-SERVER (foreign-value "CRYPT_OPTION_NET_SOCKS_SERVER" int))
(define CRYPT-OPTION-NET-SOCKS-USERNAME (foreign-value "CRYPT_OPTION_NET_SOCKS_USERNAME" int))
(define CRYPT-OPTION-NET-HTTP-PROXY (foreign-value "CRYPT_OPTION_NET_HTTP_PROXY" int))
(define CRYPT-OPTION-NET-CONNECTTIMEOUT (foreign-value "CRYPT_OPTION_NET_CONNECTTIMEOUT" int))
(define CRYPT-OPTION-NET-READTIMEOUT (foreign-value "CRYPT_OPTION_NET_READTIMEOUT" int))
(define CRYPT-OPTION-NET-WRITETIMEOUT (foreign-value "CRYPT_OPTION_NET_WRITETIMEOUT" int))
(define CRYPT-OPTION-MISC-ASYNCINIT (foreign-value "CRYPT_OPTION_MISC_ASYNCINIT" int))
(define CRYPT-OPTION-MISC-SIDECHANNELPROTECTION (foreign-value "CRYPT_OPTION_MISC_SIDECHANNELPROTECTION" int))
(define CRYPT-OPTION-CONFIGCHANGED (foreign-value "CRYPT_OPTION_CONFIGCHANGED" int))
(define CRYPT-OPTION-SELFTESTOK (foreign-value "CRYPT_OPTION_SELFTESTOK" int))
(define CRYPT-CTXINFO-ALGO (foreign-value "CRYPT_CTXINFO_ALGO" int))
(define CRYPT-CTXINFO-MODE (foreign-value "CRYPT_CTXINFO_MODE" int))
(define CRYPT-CTXINFO-NAME-ALGO (foreign-value "CRYPT_CTXINFO_NAME_ALGO" int))
(define CRYPT-CTXINFO-NAME-MODE (foreign-value "CRYPT_CTXINFO_NAME_MODE" int))
(define CRYPT-CTXINFO-KEYSIZE (foreign-value "CRYPT_CTXINFO_KEYSIZE" int))
(define CRYPT-CTXINFO-BLOCKSIZE (foreign-value "CRYPT_CTXINFO_BLOCKSIZE" int))
(define CRYPT-CTXINFO-IVSIZE (foreign-value "CRYPT_CTXINFO_IVSIZE" int))
(define CRYPT-CTXINFO-KEYING-ALGO (foreign-value "CRYPT_CTXINFO_KEYING_ALGO" int))
(define CRYPT-CTXINFO-KEYING-ITERATIONS (foreign-value "CRYPT_CTXINFO_KEYING_ITERATIONS" int))
(define CRYPT-CTXINFO-KEYING-SALT (foreign-value "CRYPT_CTXINFO_KEYING_SALT" int))
(define CRYPT-CTXINFO-KEYING-VALUE (foreign-value "CRYPT_CTXINFO_KEYING_VALUE" int))
(define CRYPT-CTXINFO-KEY (foreign-value "CRYPT_CTXINFO_KEY" int))
(define CRYPT-CTXINFO-KEY-COMPONENTS (foreign-value "CRYPT_CTXINFO_KEY_COMPONENTS" int))
(define CRYPT-CTXINFO-IV (foreign-value "CRYPT_CTXINFO_IV" int))
(define CRYPT-CTXINFO-HASHVALUE (foreign-value "CRYPT_CTXINFO_HASHVALUE" int))
(define CRYPT-CTXINFO-LABEL (foreign-value "CRYPT_CTXINFO_LABEL" int))
(define CRYPT-CERTINFO-SELFSIGNED (foreign-value "CRYPT_CERTINFO_SELFSIGNED" int))
(define CRYPT-CERTINFO-IMMUTABLE (foreign-value "CRYPT_CERTINFO_IMMUTABLE" int))
(define CRYPT-CERTINFO-XYZZY (foreign-value "CRYPT_CERTINFO_XYZZY" int))
(define CRYPT-CERTINFO-CERTTYPE (foreign-value "CRYPT_CERTINFO_CERTTYPE" int))
(define CRYPT-CERTINFO-FINGERPRINT (foreign-value "CRYPT_CERTINFO_FINGERPRINT" int))
(define CRYPT-CERTINFO-FINGERPRINT-MD5 (foreign-value "CRYPT_CERTINFO_FINGERPRINT_MD5" int))
(define CRYPT-CERTINFO-FINGERPRINT-SHA (foreign-value "CRYPT_CERTINFO_FINGERPRINT_SHA" int))
(define CRYPT-CERTINFO-CURRENT-CERTIFICATE (foreign-value "CRYPT_CERTINFO_CURRENT_CERTIFICATE" int))
(define CRYPT-CERTINFO-CURRENT-EXTENSION (foreign-value "CRYPT_CERTINFO_CURRENT_EXTENSION" int))
(define CRYPT-CERTINFO-CURRENT-FIELD (foreign-value "CRYPT_CERTINFO_CURRENT_FIELD" int))
(define CRYPT-CERTINFO-CURRENT-COMPONENT (foreign-value "CRYPT_CERTINFO_CURRENT_COMPONENT" int))
(define CRYPT-CERTINFO-TRUSTED-USAGE (foreign-value "CRYPT_CERTINFO_TRUSTED_USAGE" int))
(define CRYPT-CERTINFO-TRUSTED-IMPLICIT (foreign-value "CRYPT_CERTINFO_TRUSTED_IMPLICIT" int))
(define CRYPT-CERTINFO-SIGNATURELEVEL (foreign-value "CRYPT_CERTINFO_SIGNATURELEVEL" int))
(define CRYPT-CERTINFO-VERSION (foreign-value "CRYPT_CERTINFO_VERSION" int))
(define CRYPT-CERTINFO-SERIALNUMBER (foreign-value "CRYPT_CERTINFO_SERIALNUMBER" int))
(define CRYPT-CERTINFO-SUBJECTPUBLICKEYINFO (foreign-value "CRYPT_CERTINFO_SUBJECTPUBLICKEYINFO" int))
(define CRYPT-CERTINFO-CERTIFICATE (foreign-value "CRYPT_CERTINFO_CERTIFICATE" int))
(define CRYPT-CERTINFO-USERCERTIFICATE (foreign-value "CRYPT_CERTINFO_USERCERTIFICATE" int))
(define CRYPT-CERTINFO-CACERTIFICATE (foreign-value "CRYPT_CERTINFO_CACERTIFICATE" int))
(define CRYPT-CERTINFO-ISSUERNAME (foreign-value "CRYPT_CERTINFO_ISSUERNAME" int))
(define CRYPT-CERTINFO-VALIDFROM (foreign-value "CRYPT_CERTINFO_VALIDFROM" int))
(define CRYPT-CERTINFO-VALIDTO (foreign-value "CRYPT_CERTINFO_VALIDTO" int))
(define CRYPT-CERTINFO-SUBJECTNAME (foreign-value "CRYPT_CERTINFO_SUBJECTNAME" int))
(define CRYPT-CERTINFO-ISSUERUNIQUEID (foreign-value "CRYPT_CERTINFO_ISSUERUNIQUEID" int))
(define CRYPT-CERTINFO-SUBJECTUNIQUEID (foreign-value "CRYPT_CERTINFO_SUBJECTUNIQUEID" int))
(define CRYPT-CERTINFO-CERTREQUEST (foreign-value "CRYPT_CERTINFO_CERTREQUEST" int))
(define CRYPT-CERTINFO-THISUPDATE (foreign-value "CRYPT_CERTINFO_THISUPDATE" int))
(define CRYPT-CERTINFO-NEXTUPDATE (foreign-value "CRYPT_CERTINFO_NEXTUPDATE" int))
(define CRYPT-CERTINFO-REVOCATIONDATE (foreign-value "CRYPT_CERTINFO_REVOCATIONDATE" int))
(define CRYPT-CERTINFO-REVOCATIONSTATUS (foreign-value "CRYPT_CERTINFO_REVOCATIONSTATUS" int))
(define CRYPT-CERTINFO-CERTSTATUS (foreign-value "CRYPT_CERTINFO_CERTSTATUS" int))
(define CRYPT-CERTINFO-DN (foreign-value "CRYPT_CERTINFO_DN" int))
(define CRYPT-CERTINFO-PKIUSER-ID (foreign-value "CRYPT_CERTINFO_PKIUSER_ID" int))
(define CRYPT-CERTINFO-PKIUSER-ISSUEPASSWORD (foreign-value "CRYPT_CERTINFO_PKIUSER_ISSUEPASSWORD" int))
(define CRYPT-CERTINFO-PKIUSER-REVPASSWORD (foreign-value "CRYPT_CERTINFO_PKIUSER_REVPASSWORD" int))
(define CRYPT-CERTINFO-COUNTRYNAME (foreign-value "CRYPT_CERTINFO_COUNTRYNAME" int))
(define CRYPT-CERTINFO-STATEORPROVINCENAME (foreign-value "CRYPT_CERTINFO_STATEORPROVINCENAME" int))
(define CRYPT-CERTINFO-LOCALITYNAME (foreign-value "CRYPT_CERTINFO_LOCALITYNAME" int))
(define CRYPT-CERTINFO-ORGANIZATIONNAME (foreign-value "CRYPT_CERTINFO_ORGANIZATIONNAME" int))
(define CRYPT-CERTINFO-ORGANISATIONNAME (foreign-value "CRYPT_CERTINFO_ORGANISATIONNAME" int))
(define CRYPT-CERTINFO-ORGANIZATIONALUNITNAME (foreign-value "CRYPT_CERTINFO_ORGANIZATIONALUNITNAME" int))
(define CRYPT-CERTINFO-ORGANISATIONALUNITNAME (foreign-value "CRYPT_CERTINFO_ORGANISATIONALUNITNAME" int))
(define CRYPT-CERTINFO-COMMONNAME (foreign-value "CRYPT_CERTINFO_COMMONNAME" int))
(define CRYPT-CERTINFO-OTHERNAME-TYPEID (foreign-value "CRYPT_CERTINFO_OTHERNAME_TYPEID" int))
(define CRYPT-CERTINFO-OTHERNAME-VALUE (foreign-value "CRYPT_CERTINFO_OTHERNAME_VALUE" int))
(define CRYPT-CERTINFO-RFC822NAME (foreign-value "CRYPT_CERTINFO_RFC822NAME" int))
(define CRYPT-CERTINFO-EMAIL (foreign-value "CRYPT_CERTINFO_EMAIL" int))
(define CRYPT-CERTINFO-DNSNAME (foreign-value "CRYPT_CERTINFO_DNSNAME" int))
(define CRYPT-CERTINFO-DIRECTORYNAME (foreign-value "CRYPT_CERTINFO_DIRECTORYNAME" int))
(define CRYPT-CERTINFO-EDIPARTYNAME-NAMEASSIGNER (foreign-value "CRYPT_CERTINFO_EDIPARTYNAME_NAMEASSIGNER" int))
(define CRYPT-CERTINFO-EDIPARTYNAME-PARTYNAME (foreign-value "CRYPT_CERTINFO_EDIPARTYNAME_PARTYNAME" int))
(define CRYPT-CERTINFO-UNIFORMRESOURCEIDENTIFIER (foreign-value "CRYPT_CERTINFO_UNIFORMRESOURCEIDENTIFIER" int))
(define CRYPT-CERTINFO-IPADDRESS (foreign-value "CRYPT_CERTINFO_IPADDRESS" int))
(define CRYPT-CERTINFO-REGISTEREDID (foreign-value "CRYPT_CERTINFO_REGISTEREDID" int))
(define CRYPT-CERTINFO-CHALLENGEPASSWORD (foreign-value "CRYPT_CERTINFO_CHALLENGEPASSWORD" int))
(define CRYPT-CERTINFO-CRLEXTREASON (foreign-value "CRYPT_CERTINFO_CRLEXTREASON" int))
(define CRYPT-CERTINFO-KEYFEATURES (foreign-value "CRYPT_CERTINFO_KEYFEATURES" int))
(define CRYPT-CERTINFO-AUTHORITYINFOACCESS (foreign-value "CRYPT_CERTINFO_AUTHORITYINFOACCESS" int))
(define CRYPT-CERTINFO-AUTHORITYINFO-RTCS (foreign-value "CRYPT_CERTINFO_AUTHORITYINFO_RTCS" int))
(define CRYPT-CERTINFO-AUTHORITYINFO-OCSP (foreign-value "CRYPT_CERTINFO_AUTHORITYINFO_OCSP" int))
(define CRYPT-CERTINFO-AUTHORITYINFO-CAISSUERS (foreign-value "CRYPT_CERTINFO_AUTHORITYINFO_CAISSUERS" int))
(define CRYPT-CERTINFO-AUTHORITYINFO-CERTSTORE (foreign-value "CRYPT_CERTINFO_AUTHORITYINFO_CERTSTORE" int))
(define CRYPT-CERTINFO-AUTHORITYINFO-CRLS (foreign-value "CRYPT_CERTINFO_AUTHORITYINFO_CRLS" int))
(define CRYPT-CERTINFO-OCSP-NONCE (foreign-value "CRYPT_CERTINFO_OCSP_NONCE" int))
(define CRYPT-CERTINFO-OCSP-RESPONSE (foreign-value "CRYPT_CERTINFO_OCSP_RESPONSE" int))
(define CRYPT-CERTINFO-OCSP-RESPONSE-OCSP (foreign-value "CRYPT_CERTINFO_OCSP_RESPONSE_OCSP" int))
(define CRYPT-CERTINFO-OCSP-NOCHECK (foreign-value "CRYPT_CERTINFO_OCSP_NOCHECK" int))
(define CRYPT-CERTINFO-OCSP-ARCHIVECUTOFF (foreign-value "CRYPT_CERTINFO_OCSP_ARCHIVECUTOFF" int))
(define CRYPT-CERTINFO-SUBJECTINFOACCESS (foreign-value "CRYPT_CERTINFO_SUBJECTINFOACCESS" int))
(define CRYPT-CERTINFO-SUBJECTINFO-CAREPOSITORY (foreign-value "CRYPT_CERTINFO_SUBJECTINFO_CAREPOSITORY" int))
(define CRYPT-CERTINFO-SUBJECTINFO-TIMESTAMPING (foreign-value "CRYPT_CERTINFO_SUBJECTINFO_TIMESTAMPING" int))
(define CRYPT-CERTINFO-SIGG-DATEOFCERTGEN (foreign-value "CRYPT_CERTINFO_SIGG_DATEOFCERTGEN" int))
(define CRYPT-CERTINFO-SIGG-PROCURATION (foreign-value "CRYPT_CERTINFO_SIGG_PROCURATION" int))
(define CRYPT-CERTINFO-SIGG-PROCURE-COUNTRY (foreign-value "CRYPT_CERTINFO_SIGG_PROCURE_COUNTRY" int))
(define CRYPT-CERTINFO-SIGG-PROCURE-TYPEOFSUBSTITUTION (foreign-value "CRYPT_CERTINFO_SIGG_PROCURE_TYPEOFSUBSTITUTION" int))
(define CRYPT-CERTINFO-SIGG-PROCURE-SIGNINGFOR (foreign-value "CRYPT_CERTINFO_SIGG_PROCURE_SIGNINGFOR" int))
(define CRYPT-CERTINFO-SIGG-MONETARYLIMIT (foreign-value "CRYPT_CERTINFO_SIGG_MONETARYLIMIT" int))
(define CRYPT-CERTINFO-SIGG-MONETARY-CURRENCY (foreign-value "CRYPT_CERTINFO_SIGG_MONETARY_CURRENCY" int))
(define CRYPT-CERTINFO-SIGG-MONETARY-AMOUNT (foreign-value "CRYPT_CERTINFO_SIGG_MONETARY_AMOUNT" int))
(define CRYPT-CERTINFO-SIGG-MONETARY-EXPONENT (foreign-value "CRYPT_CERTINFO_SIGG_MONETARY_EXPONENT" int))
(define CRYPT-CERTINFO-SIGG-RESTRICTION (foreign-value "CRYPT_CERTINFO_SIGG_RESTRICTION" int))
(define CRYPT-CERTINFO-STRONGEXTRANET (foreign-value "CRYPT_CERTINFO_STRONGEXTRANET" int))
(define CRYPT-CERTINFO-STRONGEXTRANET-ZONE (foreign-value "CRYPT_CERTINFO_STRONGEXTRANET_ZONE" int))
(define CRYPT-CERTINFO-STRONGEXTRANET-ID (foreign-value "CRYPT_CERTINFO_STRONGEXTRANET_ID" int))
(define CRYPT-CERTINFO-SUBJECTDIRECTORYATTRIBUTES (foreign-value "CRYPT_CERTINFO_SUBJECTDIRECTORYATTRIBUTES" int))
(define CRYPT-CERTINFO-SUBJECTDIR-TYPE (foreign-value "CRYPT_CERTINFO_SUBJECTDIR_TYPE" int))
(define CRYPT-CERTINFO-SUBJECTDIR-VALUES (foreign-value "CRYPT_CERTINFO_SUBJECTDIR_VALUES" int))
(define CRYPT-CERTINFO-SUBJECTKEYIDENTIFIER (foreign-value "CRYPT_CERTINFO_SUBJECTKEYIDENTIFIER" int))
(define CRYPT-CERTINFO-KEYUSAGE (foreign-value "CRYPT_CERTINFO_KEYUSAGE" int))
(define CRYPT-CERTINFO-PRIVATEKEYUSAGEPERIOD (foreign-value "CRYPT_CERTINFO_PRIVATEKEYUSAGEPERIOD" int))
(define CRYPT-CERTINFO-PRIVATEKEY-NOTBEFORE (foreign-value "CRYPT_CERTINFO_PRIVATEKEY_NOTBEFORE" int))
(define CRYPT-CERTINFO-PRIVATEKEY-NOTAFTER (foreign-value "CRYPT_CERTINFO_PRIVATEKEY_NOTAFTER" int))
(define CRYPT-CERTINFO-SUBJECTALTNAME (foreign-value "CRYPT_CERTINFO_SUBJECTALTNAME" int))
(define CRYPT-CERTINFO-ISSUERALTNAME (foreign-value "CRYPT_CERTINFO_ISSUERALTNAME" int))
(define CRYPT-CERTINFO-BASICCONSTRAINTS (foreign-value "CRYPT_CERTINFO_BASICCONSTRAINTS" int))
(define CRYPT-CERTINFO-CA (foreign-value "CRYPT_CERTINFO_CA" int))
(define CRYPT-CERTINFO-AUTHORITY (foreign-value "CRYPT_CERTINFO_AUTHORITY" int))
(define CRYPT-CERTINFO-PATHLENCONSTRAINT (foreign-value "CRYPT_CERTINFO_PATHLENCONSTRAINT" int))
(define CRYPT-CERTINFO-CRLNUMBER (foreign-value "CRYPT_CERTINFO_CRLNUMBER" int))
(define CRYPT-CERTINFO-CRLREASON (foreign-value "CRYPT_CERTINFO_CRLREASON" int))
(define CRYPT-CERTINFO-HOLDINSTRUCTIONCODE (foreign-value "CRYPT_CERTINFO_HOLDINSTRUCTIONCODE" int))
(define CRYPT-CERTINFO-INVALIDITYDATE (foreign-value "CRYPT_CERTINFO_INVALIDITYDATE" int))
(define CRYPT-CERTINFO-DELTACRLINDICATOR (foreign-value "CRYPT_CERTINFO_DELTACRLINDICATOR" int))
(define CRYPT-CERTINFO-ISSUINGDISTRIBUTIONPOINT (foreign-value "CRYPT_CERTINFO_ISSUINGDISTRIBUTIONPOINT" int))
(define CRYPT-CERTINFO-ISSUINGDIST-FULLNAME (foreign-value "CRYPT_CERTINFO_ISSUINGDIST_FULLNAME" int))
(define CRYPT-CERTINFO-ISSUINGDIST-USERCERTSONLY (foreign-value "CRYPT_CERTINFO_ISSUINGDIST_USERCERTSONLY" int))
(define CRYPT-CERTINFO-ISSUINGDIST-CACERTSONLY (foreign-value "CRYPT_CERTINFO_ISSUINGDIST_CACERTSONLY" int))
(define CRYPT-CERTINFO-ISSUINGDIST-SOMEREASONSONLY (foreign-value "CRYPT_CERTINFO_ISSUINGDIST_SOMEREASONSONLY" int))
(define CRYPT-CERTINFO-ISSUINGDIST-INDIRECTCRL (foreign-value "CRYPT_CERTINFO_ISSUINGDIST_INDIRECTCRL" int))
(define CRYPT-CERTINFO-CERTIFICATEISSUER (foreign-value "CRYPT_CERTINFO_CERTIFICATEISSUER" int))
(define CRYPT-CERTINFO-NAMECONSTRAINTS (foreign-value "CRYPT_CERTINFO_NAMECONSTRAINTS" int))
(define CRYPT-CERTINFO-PERMITTEDSUBTREES (foreign-value "CRYPT_CERTINFO_PERMITTEDSUBTREES" int))
(define CRYPT-CERTINFO-EXCLUDEDSUBTREES (foreign-value "CRYPT_CERTINFO_EXCLUDEDSUBTREES" int))
(define CRYPT-CERTINFO-CRLDISTRIBUTIONPOINT (foreign-value "CRYPT_CERTINFO_CRLDISTRIBUTIONPOINT" int))
(define CRYPT-CERTINFO-CRLDIST-FULLNAME (foreign-value "CRYPT_CERTINFO_CRLDIST_FULLNAME" int))
(define CRYPT-CERTINFO-CRLDIST-REASONS (foreign-value "CRYPT_CERTINFO_CRLDIST_REASONS" int))
(define CRYPT-CERTINFO-CRLDIST-CRLISSUER (foreign-value "CRYPT_CERTINFO_CRLDIST_CRLISSUER" int))
(define CRYPT-CERTINFO-CERTIFICATEPOLICIES (foreign-value "CRYPT_CERTINFO_CERTIFICATEPOLICIES" int))
(define CRYPT-CERTINFO-CERTPOLICYID (foreign-value "CRYPT_CERTINFO_CERTPOLICYID" int))
(define CRYPT-CERTINFO-CERTPOLICY-CPSURI (foreign-value "CRYPT_CERTINFO_CERTPOLICY_CPSURI" int))
(define CRYPT-CERTINFO-CERTPOLICY-ORGANIZATION (foreign-value "CRYPT_CERTINFO_CERTPOLICY_ORGANIZATION" int))
(define CRYPT-CERTINFO-CERTPOLICY-NOTICENUMBERS (foreign-value "CRYPT_CERTINFO_CERTPOLICY_NOTICENUMBERS" int))
(define CRYPT-CERTINFO-CERTPOLICY-EXPLICITTEXT (foreign-value "CRYPT_CERTINFO_CERTPOLICY_EXPLICITTEXT" int))
(define CRYPT-CERTINFO-POLICYMAPPINGS (foreign-value "CRYPT_CERTINFO_POLICYMAPPINGS" int))
(define CRYPT-CERTINFO-ISSUERDOMAINPOLICY (foreign-value "CRYPT_CERTINFO_ISSUERDOMAINPOLICY" int))
(define CRYPT-CERTINFO-SUBJECTDOMAINPOLICY (foreign-value "CRYPT_CERTINFO_SUBJECTDOMAINPOLICY" int))
(define CRYPT-CERTINFO-AUTHORITYKEYIDENTIFIER (foreign-value "CRYPT_CERTINFO_AUTHORITYKEYIDENTIFIER" int))
(define CRYPT-CERTINFO-AUTHORITY-KEYIDENTIFIER (foreign-value "CRYPT_CERTINFO_AUTHORITY_KEYIDENTIFIER" int))
(define CRYPT-CERTINFO-AUTHORITY-CERTISSUER (foreign-value "CRYPT_CERTINFO_AUTHORITY_CERTISSUER" int))
(define CRYPT-CERTINFO-AUTHORITY-CERTSERIALNUMBER (foreign-value "CRYPT_CERTINFO_AUTHORITY_CERTSERIALNUMBER" int))
(define CRYPT-CERTINFO-POLICYCONSTRAINTS (foreign-value "CRYPT_CERTINFO_POLICYCONSTRAINTS" int))
(define CRYPT-CERTINFO-REQUIREEXPLICITPOLICY (foreign-value "CRYPT_CERTINFO_REQUIREEXPLICITPOLICY" int))
(define CRYPT-CERTINFO-INHIBITPOLICYMAPPING (foreign-value "CRYPT_CERTINFO_INHIBITPOLICYMAPPING" int))
(define CRYPT-CERTINFO-EXTKEYUSAGE (foreign-value "CRYPT_CERTINFO_EXTKEYUSAGE" int))
(define CRYPT-CERTINFO-EXTKEY-MS-INDIVIDUALCODESIGNING (foreign-value "CRYPT_CERTINFO_EXTKEY_MS_INDIVIDUALCODESIGNING" int))
(define CRYPT-CERTINFO-EXTKEY-MS-COMMERCIALCODESIGNING (foreign-value "CRYPT_CERTINFO_EXTKEY_MS_COMMERCIALCODESIGNING" int))
(define CRYPT-CERTINFO-EXTKEY-MS-CERTTRUSTLISTSIGNING (foreign-value "CRYPT_CERTINFO_EXTKEY_MS_CERTTRUSTLISTSIGNING" int))
(define CRYPT-CERTINFO-EXTKEY-MS-TIMESTAMPSIGNING (foreign-value "CRYPT_CERTINFO_EXTKEY_MS_TIMESTAMPSIGNING" int))
(define CRYPT-CERTINFO-EXTKEY-MS-SERVERGATEDCRYPTO (foreign-value "CRYPT_CERTINFO_EXTKEY_MS_SERVERGATEDCRYPTO" int))
(define CRYPT-CERTINFO-EXTKEY-MS-ENCRYPTEDFILESYSTEM (foreign-value "CRYPT_CERTINFO_EXTKEY_MS_ENCRYPTEDFILESYSTEM" int))
(define CRYPT-CERTINFO-EXTKEY-SERVERAUTH (foreign-value "CRYPT_CERTINFO_EXTKEY_SERVERAUTH" int))
(define CRYPT-CERTINFO-EXTKEY-CLIENTAUTH (foreign-value "CRYPT_CERTINFO_EXTKEY_CLIENTAUTH" int))
(define CRYPT-CERTINFO-EXTKEY-CODESIGNING (foreign-value "CRYPT_CERTINFO_EXTKEY_CODESIGNING" int))
(define CRYPT-CERTINFO-EXTKEY-EMAILPROTECTION (foreign-value "CRYPT_CERTINFO_EXTKEY_EMAILPROTECTION" int))
(define CRYPT-CERTINFO-EXTKEY-IPSECENDSYSTEM (foreign-value "CRYPT_CERTINFO_EXTKEY_IPSECENDSYSTEM" int))
(define CRYPT-CERTINFO-EXTKEY-IPSECTUNNEL (foreign-value "CRYPT_CERTINFO_EXTKEY_IPSECTUNNEL" int))
(define CRYPT-CERTINFO-EXTKEY-IPSECUSER (foreign-value "CRYPT_CERTINFO_EXTKEY_IPSECUSER" int))
(define CRYPT-CERTINFO-EXTKEY-TIMESTAMPING (foreign-value "CRYPT_CERTINFO_EXTKEY_TIMESTAMPING" int))
(define CRYPT-CERTINFO-EXTKEY-OCSPSIGNING (foreign-value "CRYPT_CERTINFO_EXTKEY_OCSPSIGNING" int))
(define CRYPT-CERTINFO-EXTKEY-DIRECTORYSERVICE (foreign-value "CRYPT_CERTINFO_EXTKEY_DIRECTORYSERVICE" int))
(define CRYPT-CERTINFO-EXTKEY-NS-SERVERGATEDCRYPTO (foreign-value "CRYPT_CERTINFO_EXTKEY_NS_SERVERGATEDCRYPTO" int))
(define CRYPT-CERTINFO-EXTKEY-VS-SERVERGATEDCRYPTO-CA (foreign-value "CRYPT_CERTINFO_EXTKEY_VS_SERVERGATEDCRYPTO_CA" int))
(define CRYPT-CERTINFO-FRESHESTCRL (foreign-value "CRYPT_CERTINFO_FRESHESTCRL" int))
(define CRYPT-CERTINFO-FRESHESTCRL-FULLNAME (foreign-value "CRYPT_CERTINFO_FRESHESTCRL_FULLNAME" int))
(define CRYPT-CERTINFO-FRESHESTCRL-REASONS (foreign-value "CRYPT_CERTINFO_FRESHESTCRL_REASONS" int))
(define CRYPT-CERTINFO-FRESHESTCRL-CRLISSUER (foreign-value "CRYPT_CERTINFO_FRESHESTCRL_CRLISSUER" int))
(define CRYPT-CERTINFO-INHIBITANYPOLICY (foreign-value "CRYPT_CERTINFO_INHIBITANYPOLICY" int))
(define CRYPT-CERTINFO-NS-CERTTYPE (foreign-value "CRYPT_CERTINFO_NS_CERTTYPE" int))
(define CRYPT-CERTINFO-NS-BASEURL (foreign-value "CRYPT_CERTINFO_NS_BASEURL" int))
(define CRYPT-CERTINFO-NS-REVOCATIONURL (foreign-value "CRYPT_CERTINFO_NS_REVOCATIONURL" int))
(define CRYPT-CERTINFO-NS-CAREVOCATIONURL (foreign-value "CRYPT_CERTINFO_NS_CAREVOCATIONURL" int))
(define CRYPT-CERTINFO-NS-CERTRENEWALURL (foreign-value "CRYPT_CERTINFO_NS_CERTRENEWALURL" int))
(define CRYPT-CERTINFO-NS-CAPOLICYURL (foreign-value "CRYPT_CERTINFO_NS_CAPOLICYURL" int))
(define CRYPT-CERTINFO-NS-SSLSERVERNAME (foreign-value "CRYPT_CERTINFO_NS_SSLSERVERNAME" int))
(define CRYPT-CERTINFO-NS-COMMENT (foreign-value "CRYPT_CERTINFO_NS_COMMENT" int))
(define CRYPT-CERTINFO-SET-HASHEDROOTKEY (foreign-value "CRYPT_CERTINFO_SET_HASHEDROOTKEY" int))
(define CRYPT-CERTINFO-SET-ROOTKEYTHUMBPRINT (foreign-value "CRYPT_CERTINFO_SET_ROOTKEYTHUMBPRINT" int))
(define CRYPT-CERTINFO-SET-CERTIFICATETYPE (foreign-value "CRYPT_CERTINFO_SET_CERTIFICATETYPE" int))
(define CRYPT-CERTINFO-SET-MERCHANTDATA (foreign-value "CRYPT_CERTINFO_SET_MERCHANTDATA" int))
(define CRYPT-CERTINFO-SET-MERID (foreign-value "CRYPT_CERTINFO_SET_MERID" int))
(define CRYPT-CERTINFO-SET-MERACQUIRERBIN (foreign-value "CRYPT_CERTINFO_SET_MERACQUIRERBIN" int))
(define CRYPT-CERTINFO-SET-MERCHANTLANGUAGE (foreign-value "CRYPT_CERTINFO_SET_MERCHANTLANGUAGE" int))
(define CRYPT-CERTINFO-SET-MERCHANTNAME (foreign-value "CRYPT_CERTINFO_SET_MERCHANTNAME" int))
(define CRYPT-CERTINFO-SET-MERCHANTCITY (foreign-value "CRYPT_CERTINFO_SET_MERCHANTCITY" int))
(define CRYPT-CERTINFO-SET-MERCHANTSTATEPROVINCE (foreign-value "CRYPT_CERTINFO_SET_MERCHANTSTATEPROVINCE" int))
(define CRYPT-CERTINFO-SET-MERCHANTPOSTALCODE (foreign-value "CRYPT_CERTINFO_SET_MERCHANTPOSTALCODE" int))
(define CRYPT-CERTINFO-SET-MERCHANTCOUNTRYNAME (foreign-value "CRYPT_CERTINFO_SET_MERCHANTCOUNTRYNAME" int))
(define CRYPT-CERTINFO-SET-MERCOUNTRY (foreign-value "CRYPT_CERTINFO_SET_MERCOUNTRY" int))
(define CRYPT-CERTINFO-SET-MERAUTHFLAG (foreign-value "CRYPT_CERTINFO_SET_MERAUTHFLAG" int))
(define CRYPT-CERTINFO-SET-CERTCARDREQUIRED (foreign-value "CRYPT_CERTINFO_SET_CERTCARDREQUIRED" int))
(define CRYPT-CERTINFO-SET-TUNNELING (foreign-value "CRYPT_CERTINFO_SET_TUNNELING" int))
(define CRYPT-CERTINFO-SET-TUNNELLING (foreign-value "CRYPT_CERTINFO_SET_TUNNELLING" int))
(define CRYPT-CERTINFO-SET-TUNNELINGFLAG (foreign-value "CRYPT_CERTINFO_SET_TUNNELINGFLAG" int))
(define CRYPT-CERTINFO-SET-TUNNELLINGFLAG (foreign-value "CRYPT_CERTINFO_SET_TUNNELLINGFLAG" int))
(define CRYPT-CERTINFO-SET-TUNNELINGALGID (foreign-value "CRYPT_CERTINFO_SET_TUNNELINGALGID" int))
(define CRYPT-CERTINFO-SET-TUNNELLINGALGID (foreign-value "CRYPT_CERTINFO_SET_TUNNELLINGALGID" int))
(define CRYPT-CERTINFO-CMS-CONTENTTYPE (foreign-value "CRYPT_CERTINFO_CMS_CONTENTTYPE" int))
(define CRYPT-CERTINFO-CMS-MESSAGEDIGEST (foreign-value "CRYPT_CERTINFO_CMS_MESSAGEDIGEST" int))
(define CRYPT-CERTINFO-CMS-SIGNINGTIME (foreign-value "CRYPT_CERTINFO_CMS_SIGNINGTIME" int))
(define CRYPT-CERTINFO-CMS-COUNTERSIGNATURE (foreign-value "CRYPT_CERTINFO_CMS_COUNTERSIGNATURE" int))
(define CRYPT-CERTINFO-CMS-SMIMECAPABILITIES (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAPABILITIES" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-3DES (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_3DES" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-AES (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_AES" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-CAST128 (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_CAST128" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-IDEA (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_IDEA" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-RC2 (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_RC2" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-RC5 (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_RC5" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-SKIPJACK (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_SKIPJACK" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-DES (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_DES" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-PREFERSIGNEDDATA (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_PREFERSIGNEDDATA" int))
(define CRYPT-CERTINFO-CMS-SMIMECAP-CANNOTDECRYPTANY (foreign-value "CRYPT_CERTINFO_CMS_SMIMECAP_CANNOTDECRYPTANY" int))
(define CRYPT-CERTINFO-CMS-RECEIPTREQUEST (foreign-value "CRYPT_CERTINFO_CMS_RECEIPTREQUEST" int))
(define CRYPT-CERTINFO-CMS-RECEIPT-CONTENTIDENTIFIER (foreign-value "CRYPT_CERTINFO_CMS_RECEIPT_CONTENTIDENTIFIER" int))
(define CRYPT-CERTINFO-CMS-RECEIPT-FROM (foreign-value "CRYPT_CERTINFO_CMS_RECEIPT_FROM" int))
(define CRYPT-CERTINFO-CMS-RECEIPT-TO (foreign-value "CRYPT_CERTINFO_CMS_RECEIPT_TO" int))
(define CRYPT-CERTINFO-CMS-SECURITYLABEL (foreign-value "CRYPT_CERTINFO_CMS_SECURITYLABEL" int))
(define CRYPT-CERTINFO-CMS-SECLABEL-CLASSIFICATION (foreign-value "CRYPT_CERTINFO_CMS_SECLABEL_CLASSIFICATION" int))
(define CRYPT-CERTINFO-CMS-SECLABEL-POLICY (foreign-value "CRYPT_CERTINFO_CMS_SECLABEL_POLICY" int))
(define CRYPT-CERTINFO-CMS-SECLABEL-PRIVACYMARK (foreign-value "CRYPT_CERTINFO_CMS_SECLABEL_PRIVACYMARK" int))
(define CRYPT-CERTINFO-CMS-SECLABEL-CATTYPE (foreign-value "CRYPT_CERTINFO_CMS_SECLABEL_CATTYPE" int))
(define CRYPT-CERTINFO-CMS-SECLABEL-CATVALUE (foreign-value "CRYPT_CERTINFO_CMS_SECLABEL_CATVALUE" int))
(define CRYPT-CERTINFO-CMS-MLEXPANSIONHISTORY (foreign-value "CRYPT_CERTINFO_CMS_MLEXPANSIONHISTORY" int))
(define CRYPT-CERTINFO-CMS-MLEXP-ENTITYIDENTIFIER (foreign-value "CRYPT_CERTINFO_CMS_MLEXP_ENTITYIDENTIFIER" int))
(define CRYPT-CERTINFO-CMS-MLEXP-TIME (foreign-value "CRYPT_CERTINFO_CMS_MLEXP_TIME" int))
(define CRYPT-CERTINFO-CMS-MLEXP-NONE (foreign-value "CRYPT_CERTINFO_CMS_MLEXP_NONE" int))
(define CRYPT-CERTINFO-CMS-MLEXP-INSTEADOF (foreign-value "CRYPT_CERTINFO_CMS_MLEXP_INSTEADOF" int))
(define CRYPT-CERTINFO-CMS-MLEXP-INADDITIONTO (foreign-value "CRYPT_CERTINFO_CMS_MLEXP_INADDITIONTO" int))
(define CRYPT-CERTINFO-CMS-CONTENTHINTS (foreign-value "CRYPT_CERTINFO_CMS_CONTENTHINTS" int))
(define CRYPT-CERTINFO-CMS-CONTENTHINT-DESCRIPTION (foreign-value "CRYPT_CERTINFO_CMS_CONTENTHINT_DESCRIPTION" int))
(define CRYPT-CERTINFO-CMS-CONTENTHINT-TYPE (foreign-value "CRYPT_CERTINFO_CMS_CONTENTHINT_TYPE" int))
(define CRYPT-CERTINFO-CMS-EQUIVALENTLABEL (foreign-value "CRYPT_CERTINFO_CMS_EQUIVALENTLABEL" int))
(define CRYPT-CERTINFO-CMS-EQVLABEL-POLICY (foreign-value "CRYPT_CERTINFO_CMS_EQVLABEL_POLICY" int))
(define CRYPT-CERTINFO-CMS-EQVLABEL-CLASSIFICATION (foreign-value "CRYPT_CERTINFO_CMS_EQVLABEL_CLASSIFICATION" int))
(define CRYPT-CERTINFO-CMS-EQVLABEL-PRIVACYMARK (foreign-value "CRYPT_CERTINFO_CMS_EQVLABEL_PRIVACYMARK" int))
(define CRYPT-CERTINFO-CMS-EQVLABEL-CATTYPE (foreign-value "CRYPT_CERTINFO_CMS_EQVLABEL_CATTYPE" int))
(define CRYPT-CERTINFO-CMS-EQVLABEL-CATVALUE (foreign-value "CRYPT_CERTINFO_CMS_EQVLABEL_CATVALUE" int))
(define CRYPT-CERTINFO-CMS-SIGNINGCERTIFICATE (foreign-value "CRYPT_CERTINFO_CMS_SIGNINGCERTIFICATE" int))
(define CRYPT-CERTINFO-CMS-SIGNINGCERT-ESSCERTID (foreign-value "CRYPT_CERTINFO_CMS_SIGNINGCERT_ESSCERTID" int))
(define CRYPT-CERTINFO-CMS-SIGNINGCERT-POLICIES (foreign-value "CRYPT_CERTINFO_CMS_SIGNINGCERT_POLICIES" int))
(define CRYPT-CERTINFO-CMS-SIGTYPEIDENTIFIER (foreign-value "CRYPT_CERTINFO_CMS_SIGTYPEIDENTIFIER" int))
(define CRYPT-CERTINFO-CMS-SIGTYPEID-ORIGINATORSIG (foreign-value "CRYPT_CERTINFO_CMS_SIGTYPEID_ORIGINATORSIG" int))
(define CRYPT-CERTINFO-CMS-SIGTYPEID-DOMAINSIG (foreign-value "CRYPT_CERTINFO_CMS_SIGTYPEID_DOMAINSIG" int))
(define CRYPT-CERTINFO-CMS-SIGTYPEID-ADDITIONALATTRIBUTES (foreign-value "CRYPT_CERTINFO_CMS_SIGTYPEID_ADDITIONALATTRIBUTES" int))
(define CRYPT-CERTINFO-CMS-SIGTYPEID-REVIEWSIG (foreign-value "CRYPT_CERTINFO_CMS_SIGTYPEID_REVIEWSIG" int))
(define CRYPT-CERTINFO-CMS-NONCE (foreign-value "CRYPT_CERTINFO_CMS_NONCE" int))
(define CRYPT-CERTINFO-SCEP-MESSAGETYPE (foreign-value "CRYPT_CERTINFO_SCEP_MESSAGETYPE" int))
(define CRYPT-CERTINFO-SCEP-PKISTATUS (foreign-value "CRYPT_CERTINFO_SCEP_PKISTATUS" int))
(define CRYPT-CERTINFO-SCEP-FAILINFO (foreign-value "CRYPT_CERTINFO_SCEP_FAILINFO" int))
(define CRYPT-CERTINFO-SCEP-SENDERNONCE (foreign-value "CRYPT_CERTINFO_SCEP_SENDERNONCE" int))
(define CRYPT-CERTINFO-SCEP-RECIPIENTNONCE (foreign-value "CRYPT_CERTINFO_SCEP_RECIPIENTNONCE" int))
(define CRYPT-CERTINFO-SCEP-TRANSACTIONID (foreign-value "CRYPT_CERTINFO_SCEP_TRANSACTIONID" int))
(define CRYPT-CERTINFO-CMS-SPCAGENCYINFO (foreign-value "CRYPT_CERTINFO_CMS_SPCAGENCYINFO" int))
(define CRYPT-CERTINFO-CMS-SPCAGENCYURL (foreign-value "CRYPT_CERTINFO_CMS_SPCAGENCYURL" int))
(define CRYPT-CERTINFO-CMS-SPCSTATEMENTTYPE (foreign-value "CRYPT_CERTINFO_CMS_SPCSTATEMENTTYPE" int))
(define CRYPT-CERTINFO-CMS-SPCSTMT-INDIVIDUALCODESIGNING (foreign-value "CRYPT_CERTINFO_CMS_SPCSTMT_INDIVIDUALCODESIGNING" int))
(define CRYPT-CERTINFO-CMS-SPCSTMT-COMMERCIALCODESIGNING (foreign-value "CRYPT_CERTINFO_CMS_SPCSTMT_COMMERCIALCODESIGNING" int))
(define CRYPT-CERTINFO-CMS-SPCOPUSINFO (foreign-value "CRYPT_CERTINFO_CMS_SPCOPUSINFO" int))
(define CRYPT-CERTINFO-CMS-SPCOPUSINFO-NAME (foreign-value "CRYPT_CERTINFO_CMS_SPCOPUSINFO_NAME" int))
(define CRYPT-CERTINFO-CMS-SPCOPUSINFO-URL (foreign-value "CRYPT_CERTINFO_CMS_SPCOPUSINFO_URL" int))
(define CRYPT-KEYINFO-QUERY (foreign-value "CRYPT_KEYINFO_QUERY" int))
(define CRYPT-KEYINFO-QUERY-REQUESTS (foreign-value "CRYPT_KEYINFO_QUERY_REQUESTS" int))
(define CRYPT-DEVINFO-INITIALISE (foreign-value "CRYPT_DEVINFO_INITIALISE" int))
(define CRYPT-DEVINFO-INITIALIZE (foreign-value "CRYPT_DEVINFO_INITIALIZE" int))
(define CRYPT-DEVINFO-AUTHENT-USER (foreign-value "CRYPT_DEVINFO_AUTHENT_USER" int))
(define CRYPT-DEVINFO-AUTHENT-SUPERVISOR (foreign-value "CRYPT_DEVINFO_AUTHENT_SUPERVISOR" int))
(define CRYPT-DEVINFO-SET-AUTHENT-USER (foreign-value "CRYPT_DEVINFO_SET_AUTHENT_USER" int))
(define CRYPT-DEVINFO-SET-AUTHENT-SUPERVISOR (foreign-value "CRYPT_DEVINFO_SET_AUTHENT_SUPERVISOR" int))
(define CRYPT-DEVINFO-ZEROISE (foreign-value "CRYPT_DEVINFO_ZEROISE" int))
(define CRYPT-DEVINFO-ZEROIZE (foreign-value "CRYPT_DEVINFO_ZEROIZE" int))
(define CRYPT-DEVINFO-LOGGEDIN (foreign-value "CRYPT_DEVINFO_LOGGEDIN" int))
(define CRYPT-DEVINFO-LABEL (foreign-value "CRYPT_DEVINFO_LABEL" int))
(define CRYPT-ENVINFO-DATASIZE (foreign-value "CRYPT_ENVINFO_DATASIZE" int))
(define CRYPT-ENVINFO-COMPRESSION (foreign-value "CRYPT_ENVINFO_COMPRESSION" int))
(define CRYPT-ENVINFO-CONTENTTYPE (foreign-value "CRYPT_ENVINFO_CONTENTTYPE" int))
(define CRYPT-ENVINFO-DETACHEDSIGNATURE (foreign-value "CRYPT_ENVINFO_DETACHEDSIGNATURE" int))
(define CRYPT-ENVINFO-SIGNATURE-RESULT (foreign-value "CRYPT_ENVINFO_SIGNATURE_RESULT" int))
(define CRYPT-ENVINFO-MAC (foreign-value "CRYPT_ENVINFO_MAC" int))
(define CRYPT-ENVINFO-PASSWORD (foreign-value "CRYPT_ENVINFO_PASSWORD" int))
(define CRYPT-ENVINFO-KEY (foreign-value "CRYPT_ENVINFO_KEY" int))
(define CRYPT-ENVINFO-SIGNATURE (foreign-value "CRYPT_ENVINFO_SIGNATURE" int))
(define CRYPT-ENVINFO-SIGNATURE-EXTRADATA (foreign-value "CRYPT_ENVINFO_SIGNATURE_EXTRADATA" int))
(define CRYPT-ENVINFO-RECIPIENT (foreign-value "CRYPT_ENVINFO_RECIPIENT" int))
(define CRYPT-ENVINFO-PUBLICKEY (foreign-value "CRYPT_ENVINFO_PUBLICKEY" int))
(define CRYPT-ENVINFO-PRIVATEKEY (foreign-value "CRYPT_ENVINFO_PRIVATEKEY" int))
(define CRYPT-ENVINFO-PRIVATEKEY-LABEL (foreign-value "CRYPT_ENVINFO_PRIVATEKEY_LABEL" int))
(define CRYPT-ENVINFO-ORIGINATOR (foreign-value "CRYPT_ENVINFO_ORIGINATOR" int))
(define CRYPT-ENVINFO-SESSIONKEY (foreign-value "CRYPT_ENVINFO_SESSIONKEY" int))
(define CRYPT-ENVINFO-HASH (foreign-value "CRYPT_ENVINFO_HASH" int))
(define CRYPT-ENVINFO-TIMESTAMP (foreign-value "CRYPT_ENVINFO_TIMESTAMP" int))
(define CRYPT-ENVINFO-KEYSET-SIGCHECK (foreign-value "CRYPT_ENVINFO_KEYSET_SIGCHECK" int))
(define CRYPT-ENVINFO-KEYSET-ENCRYPT (foreign-value "CRYPT_ENVINFO_KEYSET_ENCRYPT" int))
(define CRYPT-ENVINFO-KEYSET-DECRYPT (foreign-value "CRYPT_ENVINFO_KEYSET_DECRYPT" int))
(define CRYPT-SESSINFO-ACTIVE (foreign-value "CRYPT_SESSINFO_ACTIVE" int))
(define CRYPT-SESSINFO-CONNECTIONACTIVE (foreign-value "CRYPT_SESSINFO_CONNECTIONACTIVE" int))
(define CRYPT-SESSINFO-USERNAME (foreign-value "CRYPT_SESSINFO_USERNAME" int))
(define CRYPT-SESSINFO-PASSWORD (foreign-value "CRYPT_SESSINFO_PASSWORD" int))
(define CRYPT-SESSINFO-PRIVATEKEY (foreign-value "CRYPT_SESSINFO_PRIVATEKEY" int))
(define CRYPT-SESSINFO-AUTHRESPONSE (foreign-value "CRYPT_SESSINFO_AUTHRESPONSE" int))
(define CRYPT-SESSINFO-KEYSET (foreign-value "CRYPT_SESSINFO_KEYSET" int))
(define CRYPT-SESSINFO-SERVER-NAME (foreign-value "CRYPT_SESSINFO_SERVER_NAME" int))
(define CRYPT-SESSINFO-SERVER-PORT (foreign-value "CRYPT_SESSINFO_SERVER_PORT" int))
(define CRYPT-SESSINFO-SERVER-FINGERPRINT (foreign-value "CRYPT_SESSINFO_SERVER_FINGERPRINT" int))
(define CRYPT-SESSINFO-CLIENT-NAME (foreign-value "CRYPT_SESSINFO_CLIENT_NAME" int))
(define CRYPT-SESSINFO-CLIENT-PORT (foreign-value "CRYPT_SESSINFO_CLIENT_PORT" int))
(define CRYPT-SESSINFO-SESSION (foreign-value "CRYPT_SESSINFO_SESSION" int))
(define CRYPT-SESSINFO-NETWORKSOCKET (foreign-value "CRYPT_SESSINFO_NETWORKSOCKET" int))
(define CRYPT-SESSINFO-VERSION (foreign-value "CRYPT_SESSINFO_VERSION" int))
(define CRYPT-SESSINFO-REQUEST (foreign-value "CRYPT_SESSINFO_REQUEST" int))
(define CRYPT-SESSINFO-RESPONSE (foreign-value "CRYPT_SESSINFO_RESPONSE" int))
(define CRYPT-SESSINFO-CACERTIFICATE (foreign-value "CRYPT_SESSINFO_CACERTIFICATE" int))
(define CRYPT-SESSINFO-TSP-MSGIMPRINT (foreign-value "CRYPT_SESSINFO_TSP_MSGIMPRINT" int))
(define CRYPT-SESSINFO-CMP-REQUESTTYPE (foreign-value "CRYPT_SESSINFO_CMP_REQUESTTYPE" int))
(define CRYPT-SESSINFO-CMP-PKIBOOT (foreign-value "CRYPT_SESSINFO_CMP_PKIBOOT" int))
(define CRYPT-SESSINFO-CMP-PRIVKEYSET (foreign-value "CRYPT_SESSINFO_CMP_PRIVKEYSET" int))
(define CRYPT-SESSINFO-SSH-CHANNEL (foreign-value "CRYPT_SESSINFO_SSH_CHANNEL" int))
(define CRYPT-SESSINFO-SSH-CHANNEL-TYPE (foreign-value "CRYPT_SESSINFO_SSH_CHANNEL_TYPE" int))
(define CRYPT-SESSINFO-SSH-CHANNEL-ARG1 (foreign-value "CRYPT_SESSINFO_SSH_CHANNEL_ARG1" int))
(define CRYPT-SESSINFO-SSH-CHANNEL-ARG2 (foreign-value "CRYPT_SESSINFO_SSH_CHANNEL_ARG2" int))
(define CRYPT-SESSINFO-SSH-CHANNEL-ACTIVE (foreign-value "CRYPT_SESSINFO_SSH_CHANNEL_ACTIVE" int))
(define CRYPT-USERINFO-PASSWORD (foreign-value "CRYPT_USERINFO_PASSWORD" int))
(define CRYPT-USERINFO-CAKEY-CERTSIGN (foreign-value "CRYPT_USERINFO_CAKEY_CERTSIGN" int))
(define CRYPT-USERINFO-CAKEY-CRLSIGN (foreign-value "CRYPT_USERINFO_CAKEY_CRLSIGN" int))
(define CRYPT-USERINFO-CAKEY-RTCSSIGN (foreign-value "CRYPT_USERINFO_CAKEY_RTCSSIGN" int))
(define CRYPT-USERINFO-CAKEY-OCSPSIGN (foreign-value "CRYPT_USERINFO_CAKEY_OCSPSIGN" int))
(define CRYPT-KEYUSAGE-NONE (foreign-value "CRYPT_KEYUSAGE_NONE" int))
(define CRYPT-KEYUSAGE-DIGITALSIGNATURE (foreign-value "CRYPT_KEYUSAGE_DIGITALSIGNATURE" int))
(define CRYPT-KEYUSAGE-NONREPUDIATION (foreign-value "CRYPT_KEYUSAGE_NONREPUDIATION" int))
(define CRYPT-KEYUSAGE-KEYENCIPHERMENT (foreign-value "CRYPT_KEYUSAGE_KEYENCIPHERMENT" int))
(define CRYPT-KEYUSAGE-DATAENCIPHERMENT (foreign-value "CRYPT_KEYUSAGE_DATAENCIPHERMENT" int))
(define CRYPT-KEYUSAGE-KEYAGREEMENT (foreign-value "CRYPT_KEYUSAGE_KEYAGREEMENT" int))
(define CRYPT-KEYUSAGE-KEYCERTSIGN (foreign-value "CRYPT_KEYUSAGE_KEYCERTSIGN" int))
(define CRYPT-KEYUSAGE-CRLSIGN (foreign-value "CRYPT_KEYUSAGE_CRLSIGN" int))
(define CRYPT-KEYUSAGE-ENCIPHERONLY (foreign-value "CRYPT_KEYUSAGE_ENCIPHERONLY" int))
(define CRYPT-KEYUSAGE-DECIPHERONLY (foreign-value "CRYPT_KEYUSAGE_DECIPHERONLY" int))
(define CRYPT-KEYUSAGE-LAST (foreign-value "CRYPT_KEYUSAGE_LAST" int))
(define CRYPT-CRLREASON-UNSPECIFIED (foreign-value "CRYPT_CRLREASON_UNSPECIFIED" int))
(define CRYPT-CRLREASON-KEYCOMPROMISE (foreign-value "CRYPT_CRLREASON_KEYCOMPROMISE" int))
(define CRYPT-CRLREASON-CACOMPROMISE (foreign-value "CRYPT_CRLREASON_CACOMPROMISE" int))
(define CRYPT-CRLREASON-AFFILIATIONCHANGED (foreign-value "CRYPT_CRLREASON_AFFILIATIONCHANGED" int))
(define CRYPT-CRLREASON-SUPERSEDED (foreign-value "CRYPT_CRLREASON_SUPERSEDED" int))
(define CRYPT-CRLREASON-CESSATIONOFOPERATION (foreign-value "CRYPT_CRLREASON_CESSATIONOFOPERATION" int))
(define CRYPT-CRLREASON-CERTIFICATEHOLD (foreign-value "CRYPT_CRLREASON_CERTIFICATEHOLD" int))
(define CRYPT-CRLREASON-REMOVEFROMCRL (foreign-value "CRYPT_CRLREASON_REMOVEFROMCRL" int))
(define CRYPT-CRLREASON-LAST (foreign-value "CRYPT_CRLREASON_LAST" int))
(define CRYPT-CRLREASON-NEVERVALID (foreign-value "CRYPT_CRLREASON_NEVERVALID" int))
(define CRYPT-CRLEXTREASON-LAST (foreign-value "CRYPT_CRLEXTREASON_LAST" int))
(define CRYPT-CRLREASONFLAG-UNUSED (foreign-value "CRYPT_CRLREASONFLAG_UNUSED" int))
(define CRYPT-CRLREASONFLAG-KEYCOMPROMISE (foreign-value "CRYPT_CRLREASONFLAG_KEYCOMPROMISE" int))
(define CRYPT-CRLREASONFLAG-CACOMPROMISE (foreign-value "CRYPT_CRLREASONFLAG_CACOMPROMISE" int))
(define CRYPT-CRLREASONFLAG-AFFILIATIONCHANGED (foreign-value "CRYPT_CRLREASONFLAG_AFFILIATIONCHANGED" int))
(define CRYPT-CRLREASONFLAG-SUPERSEDED (foreign-value "CRYPT_CRLREASONFLAG_SUPERSEDED" int))
(define CRYPT-CRLREASONFLAG-CESSATIONOFOPERATION (foreign-value "CRYPT_CRLREASONFLAG_CESSATIONOFOPERATION" int))
(define CRYPT-CRLREASONFLAG-CERTIFICATEHOLD (foreign-value "CRYPT_CRLREASONFLAG_CERTIFICATEHOLD" int))
(define CRYPT-CRLREASONFLAG-LAST (foreign-value "CRYPT_CRLREASONFLAG_LAST" int))
(define CRYPT-HOLDINSTRUCTION-NONE (foreign-value "CRYPT_HOLDINSTRUCTION_NONE" int))
(define CRYPT-HOLDINSTRUCTION-CALLISSUER (foreign-value "CRYPT_HOLDINSTRUCTION_CALLISSUER" int))
(define CRYPT-HOLDINSTRUCTION-REJECT (foreign-value "CRYPT_HOLDINSTRUCTION_REJECT" int))
(define CRYPT-HOLDINSTRUCTION-PICKUPTOKEN (foreign-value "CRYPT_HOLDINSTRUCTION_PICKUPTOKEN" int))
(define CRYPT-HOLDINSTRUCTION-LAST (foreign-value "CRYPT_HOLDINSTRUCTION_LAST" int))
(define CRYPT-COMPLIANCELEVEL-OBLIVIOUS (foreign-value "CRYPT_COMPLIANCELEVEL_OBLIVIOUS" int))
(define CRYPT-COMPLIANCELEVEL-REDUCED (foreign-value "CRYPT_COMPLIANCELEVEL_REDUCED" int))
(define CRYPT-COMPLIANCELEVEL-STANDARD (foreign-value "CRYPT_COMPLIANCELEVEL_STANDARD" int))
(define CRYPT-COMPLIANCELEVEL-PKIX-PARTIAL (foreign-value "CRYPT_COMPLIANCELEVEL_PKIX_PARTIAL" int))
(define CRYPT-COMPLIANCELEVEL-PKIX-FULL (foreign-value "CRYPT_COMPLIANCELEVEL_PKIX_FULL" int))
(define CRYPT-COMPLIANCELEVEL-LAST (foreign-value "CRYPT_COMPLIANCELEVEL_LAST" int))
(define CRYPT-NS-CERTTYPE-SSLCLIENT (foreign-value "CRYPT_NS_CERTTYPE_SSLCLIENT" int))
(define CRYPT-NS-CERTTYPE-SSLSERVER (foreign-value "CRYPT_NS_CERTTYPE_SSLSERVER" int))
(define CRYPT-NS-CERTTYPE-SMIME (foreign-value "CRYPT_NS_CERTTYPE_SMIME" int))
(define CRYPT-NS-CERTTYPE-OBJECTSIGNING (foreign-value "CRYPT_NS_CERTTYPE_OBJECTSIGNING" int))
(define CRYPT-NS-CERTTYPE-RESERVED (foreign-value "CRYPT_NS_CERTTYPE_RESERVED" int))
(define CRYPT-NS-CERTTYPE-SSLCA (foreign-value "CRYPT_NS_CERTTYPE_SSLCA" int))
(define CRYPT-NS-CERTTYPE-SMIMECA (foreign-value "CRYPT_NS_CERTTYPE_SMIMECA" int))
(define CRYPT-NS-CERTTYPE-OBJECTSIGNINGCA (foreign-value "CRYPT_NS_CERTTYPE_OBJECTSIGNINGCA" int))
(define CRYPT-NS-CERTTYPE-LAST (foreign-value "CRYPT_NS_CERTTYPE_LAST" int))
(define CRYPT-SET-CERTTYPE-CARD (foreign-value "CRYPT_SET_CERTTYPE_CARD" int))
(define CRYPT-SET-CERTTYPE-MER (foreign-value "CRYPT_SET_CERTTYPE_MER" int))
(define CRYPT-SET-CERTTYPE-PGWY (foreign-value "CRYPT_SET_CERTTYPE_PGWY" int))
(define CRYPT-SET-CERTTYPE-CCA (foreign-value "CRYPT_SET_CERTTYPE_CCA" int))
(define CRYPT-SET-CERTTYPE-MCA (foreign-value "CRYPT_SET_CERTTYPE_MCA" int))
(define CRYPT-SET-CERTTYPE-PCA (foreign-value "CRYPT_SET_CERTTYPE_PCA" int))
(define CRYPT-SET-CERTTYPE-GCA (foreign-value "CRYPT_SET_CERTTYPE_GCA" int))
(define CRYPT-SET-CERTTYPE-BCA (foreign-value "CRYPT_SET_CERTTYPE_BCA" int))
(define CRYPT-SET-CERTTYPE-RCA (foreign-value "CRYPT_SET_CERTTYPE_RCA" int))
(define CRYPT-SET-CERTTYPE-ACQ (foreign-value "CRYPT_SET_CERTTYPE_ACQ" int))
(define CRYPT-SET-CERTTYPE-LAST (foreign-value "CRYPT_SET_CERTTYPE_LAST" int))
(define CRYPT-CONTENT-NONE (foreign-value "CRYPT_CONTENT_NONE" int))
(define CRYPT-CONTENT-DATA (foreign-value "CRYPT_CONTENT_DATA" int))
(define CRYPT-CONTENT-SIGNEDDATA (foreign-value "CRYPT_CONTENT_SIGNEDDATA" int))
(define CRYPT-CONTENT-ENVELOPEDDATA (foreign-value "CRYPT_CONTENT_ENVELOPEDDATA" int))
(define CRYPT-CONTENT-SIGNEDANDENVELOPEDDATA (foreign-value "CRYPT_CONTENT_SIGNEDANDENVELOPEDDATA" int))
(define CRYPT-CONTENT-DIGESTEDDATA (foreign-value "CRYPT_CONTENT_DIGESTEDDATA" int))
(define CRYPT-CONTENT-ENCRYPTEDDATA (foreign-value "CRYPT_CONTENT_ENCRYPTEDDATA" int))
(define CRYPT-CONTENT-COMPRESSEDDATA (foreign-value "CRYPT_CONTENT_COMPRESSEDDATA" int))
(define CRYPT-CONTENT-TSTINFO (foreign-value "CRYPT_CONTENT_TSTINFO" int))
(define CRYPT-CONTENT-SPCINDIRECTDATACONTEXT (foreign-value "CRYPT_CONTENT_SPCINDIRECTDATACONTEXT" int))
(define CRYPT-CONTENT-RTCSREQUEST (foreign-value "CRYPT_CONTENT_RTCSREQUEST" int))
(define CRYPT-CONTENT-RTCSRESPONSE (foreign-value "CRYPT_CONTENT_RTCSRESPONSE" int))
(define CRYPT-CONTENT-RTCSRESPONSE-EXT (foreign-value "CRYPT_CONTENT_RTCSRESPONSE_EXT" int))
(define CRYPT-CONTENT-LAST (foreign-value "CRYPT_CONTENT_LAST" int))
(define CRYPT-CLASSIFICATION-UNMARKED (foreign-value "CRYPT_CLASSIFICATION_UNMARKED" int))
(define CRYPT-CLASSIFICATION-UNCLASSIFIED (foreign-value "CRYPT_CLASSIFICATION_UNCLASSIFIED" int))
(define CRYPT-CLASSIFICATION-RESTRICTED (foreign-value "CRYPT_CLASSIFICATION_RESTRICTED" int))
(define CRYPT-CLASSIFICATION-CONFIDENTIAL (foreign-value "CRYPT_CLASSIFICATION_CONFIDENTIAL" int))
(define CRYPT-CLASSIFICATION-SECRET (foreign-value "CRYPT_CLASSIFICATION_SECRET" int))
(define CRYPT-CLASSIFICATION-TOP-SECRET (foreign-value "CRYPT_CLASSIFICATION_TOP_SECRET" int))
(define CRYPT-CLASSIFICATION-LAST (foreign-value "CRYPT_CLASSIFICATION_LAST" int))
(define CRYPT-CERTSTATUS-VALID (foreign-value "CRYPT_CERTSTATUS_VALID" int))
(define CRYPT-CERTSTATUS-NOTVALID (foreign-value "CRYPT_CERTSTATUS_NOTVALID" int))
(define CRYPT-CERTSTATUS-NONAUTHORITATIVE (foreign-value "CRYPT_CERTSTATUS_NONAUTHORITATIVE" int))
(define CRYPT-CERTSTATUS-UNKNOWN (foreign-value "CRYPT_CERTSTATUS_UNKNOWN" int))
(define CRYPT-OCSPSTATUS-NOTREVOKED (foreign-value "CRYPT_OCSPSTATUS_NOTREVOKED" int))
(define CRYPT-OCSPSTATUS-REVOKED (foreign-value "CRYPT_OCSPSTATUS_REVOKED" int))
(define CRYPT-OCSPSTATUS-UNKNOWN (foreign-value "CRYPT_OCSPSTATUS_UNKNOWN" int))
(define CRYPT-SIGNATURELEVEL-NONE (foreign-value "CRYPT_SIGNATURELEVEL_NONE" int))
(define CRYPT-SIGNATURELEVEL-SIGNERCERT (foreign-value "CRYPT_SIGNATURELEVEL_SIGNERCERT" int))
(define CRYPT-SIGNATURELEVEL-ALL (foreign-value "CRYPT_SIGNATURELEVEL_ALL" int))
(define CRYPT-SIGNATURELEVEL-LAST (foreign-value "CRYPT_SIGNATURELEVEL_LAST" int))
(define CRYPT-CERTFORMAT-NONE (foreign-value "CRYPT_CERTFORMAT_NONE" int))
(define CRYPT-CERTFORMAT-CERTIFICATE (foreign-value "CRYPT_CERTFORMAT_CERTIFICATE" int))
(define CRYPT-CERTFORMAT-CERTCHAIN (foreign-value "CRYPT_CERTFORMAT_CERTCHAIN" int))
(define CRYPT-CERTFORMAT-TEXT-CERTIFICATE (foreign-value "CRYPT_CERTFORMAT_TEXT_CERTIFICATE" int))
(define CRYPT-CERTFORMAT-TEXT-CERTCHAIN (foreign-value "CRYPT_CERTFORMAT_TEXT_CERTCHAIN" int))
(define CRYPT-CERTFORMAT-XML-CERTIFICATE (foreign-value "CRYPT_CERTFORMAT_XML_CERTIFICATE" int))
(define CRYPT-CERTFORMAT-XML-CERTCHAIN (foreign-value "CRYPT_CERTFORMAT_XML_CERTCHAIN" int))
(define CRYPT-CERTFORMAT-LAST (foreign-value "CRYPT_CERTFORMAT_LAST" int))
(define CRYPT-REQUESTTYPE-NONE (foreign-value "CRYPT_REQUESTTYPE_NONE" int))
(define CRYPT-REQUESTTYPE-INITIALISATION (foreign-value "CRYPT_REQUESTTYPE_INITIALISATION" int))
(define CRYPT-REQUESTTYPE-INITIALIZATION (foreign-value "CRYPT_REQUESTTYPE_INITIALIZATION" int))
(define CRYPT-REQUESTTYPE-CERTIFICATE (foreign-value "CRYPT_REQUESTTYPE_CERTIFICATE" int))
(define CRYPT-REQUESTTYPE-KEYUPDATE (foreign-value "CRYPT_REQUESTTYPE_KEYUPDATE" int))
(define CRYPT-REQUESTTYPE-REVOCATION (foreign-value "CRYPT_REQUESTTYPE_REVOCATION" int))
(define CRYPT-REQUESTTYPE-PKIBOOT (foreign-value "CRYPT_REQUESTTYPE_PKIBOOT" int))
(define CRYPT-REQUESTTYPE-LAST (foreign-value "CRYPT_REQUESTTYPE_LAST" int))
(define CRYPT-KEYID-NONE (foreign-value "CRYPT_KEYID_NONE" int))
(define CRYPT-KEYID-NAME (foreign-value "CRYPT_KEYID_NAME" int))
(define CRYPT-KEYID-EMAIL (foreign-value "CRYPT_KEYID_EMAIL" int))
(define CRYPT-KEYID-LAST (foreign-value "CRYPT_KEYID_LAST" int))
(define CRYPT-OBJECT-NONE (foreign-value "CRYPT_OBJECT_NONE" int))
(define CRYPT-OBJECT-ENCRYPTED-KEY (foreign-value "CRYPT_OBJECT_ENCRYPTED_KEY" int))
(define CRYPT-OBJECT-PKCENCRYPTED-KEY (foreign-value "CRYPT_OBJECT_PKCENCRYPTED_KEY" int))
(define CRYPT-OBJECT-KEYAGREEMENT (foreign-value "CRYPT_OBJECT_KEYAGREEMENT" int))
(define CRYPT-OBJECT-SIGNATURE (foreign-value "CRYPT_OBJECT_SIGNATURE" int))
(define CRYPT-OBJECT-LAST (foreign-value "CRYPT_OBJECT_LAST" int))
(define CRYPT-ERRTYPE-NONE (foreign-value "CRYPT_ERRTYPE_NONE" int))
(define CRYPT-ERRTYPE-ATTR-SIZE (foreign-value "CRYPT_ERRTYPE_ATTR_SIZE" int))
(define CRYPT-ERRTYPE-ATTR-VALUE (foreign-value "CRYPT_ERRTYPE_ATTR_VALUE" int))
(define CRYPT-ERRTYPE-ATTR-ABSENT (foreign-value "CRYPT_ERRTYPE_ATTR_ABSENT" int))
(define CRYPT-ERRTYPE-ATTR-PRESENT (foreign-value "CRYPT_ERRTYPE_ATTR_PRESENT" int))
(define CRYPT-ERRTYPE-CONSTRAINT (foreign-value "CRYPT_ERRTYPE_CONSTRAINT" int))
(define CRYPT-ERRTYPE-ISSUERCONSTRAINT (foreign-value "CRYPT_ERRTYPE_ISSUERCONSTRAINT" int))
(define CRYPT-ERRTYPE-LAST (foreign-value "CRYPT_ERRTYPE_LAST" int))
(define CRYPT-CERTACTION-NONE (foreign-value "CRYPT_CERTACTION_NONE" int))
(define CRYPT-CERTACTION-CREATE (foreign-value "CRYPT_CERTACTION_CREATE" int))
(define CRYPT-CERTACTION-CONNECT (foreign-value "CRYPT_CERTACTION_CONNECT" int))
(define CRYPT-CERTACTION-DISCONNECT (foreign-value "CRYPT_CERTACTION_DISCONNECT" int))
(define CRYPT-CERTACTION-ERROR (foreign-value "CRYPT_CERTACTION_ERROR" int))
(define CRYPT-CERTACTION-ADDUSER (foreign-value "CRYPT_CERTACTION_ADDUSER" int))
(define CRYPT-CERTACTION-REQUEST-CERT (foreign-value "CRYPT_CERTACTION_REQUEST_CERT" int))
(define CRYPT-CERTACTION-REQUEST-RENEWAL (foreign-value "CRYPT_CERTACTION_REQUEST_RENEWAL" int))
(define CRYPT-CERTACTION-REQUEST-REVOCATION (foreign-value "CRYPT_CERTACTION_REQUEST_REVOCATION" int))
(define CRYPT-CERTACTION-CERT-CREATION (foreign-value "CRYPT_CERTACTION_CERT_CREATION" int))
(define CRYPT-CERTACTION-CERT-CREATION-COMPLETE (foreign-value "CRYPT_CERTACTION_CERT_CREATION_COMPLETE" int))
(define CRYPT-CERTACTION-CERT-CREATION-DROP (foreign-value "CRYPT_CERTACTION_CERT_CREATION_DROP" int))
(define CRYPT-CERTACTION-CERT-CREATION-REVERSE (foreign-value "CRYPT_CERTACTION_CERT_CREATION_REVERSE" int))
(define CRYPT-CERTACTION-RESTART-CLEANUP (foreign-value "CRYPT_CERTACTION_RESTART_CLEANUP" int))
(define CRYPT-CERTACTION-RESTART-REVOKE-CERT (foreign-value "CRYPT_CERTACTION_RESTART_REVOKE_CERT" int))
(define CRYPT-CERTACTION-ISSUE-CERT (foreign-value "CRYPT_CERTACTION_ISSUE_CERT" int))
(define CRYPT-CERTACTION-ISSUE-CRL (foreign-value "CRYPT_CERTACTION_ISSUE_CRL" int))
(define CRYPT-CERTACTION-REVOKE-CERT (foreign-value "CRYPT_CERTACTION_REVOKE_CERT" int))
(define CRYPT-CERTACTION-EXPIRE-CERT (foreign-value "CRYPT_CERTACTION_EXPIRE_CERT" int))
(define CRYPT-CERTACTION-CLEANUP (foreign-value "CRYPT_CERTACTION_CLEANUP" int))
(define CRYPT-CERTACTION-LAST (foreign-value "CRYPT_CERTACTION_LAST" int))
(define CRYPT-MAX-KEYSIZE (foreign-value "CRYPT_MAX_KEYSIZE" int))
(define CRYPT-MAX-IVSIZE (foreign-value "CRYPT_MAX_IVSIZE" int))
(define CRYPT-MAX-PKCSIZE (foreign-value "CRYPT_MAX_PKCSIZE" int))
(define CRYPT-MAX-HASHSIZE (foreign-value "CRYPT_MAX_HASHSIZE" int))
(define CRYPT-MAX-TEXTSIZE (foreign-value "CRYPT_MAX_TEXTSIZE" int))
(define CRYPT-USE-DEFAULT (foreign-value "CRYPT_USE_DEFAULT" int))
(define CRYPT-UNUSED (foreign-value "CRYPT_UNUSED" int))
(define CRYPT-KEYTYPE-PRIVATE (foreign-value "CRYPT_KEYTYPE_PRIVATE" int))
(define CRYPT-KEYTYPE-PUBLIC (foreign-value "CRYPT_KEYTYPE_PUBLIC" int))
(define CRYPT-RANDOM-FASTPOLL (foreign-value "CRYPT_RANDOM_FASTPOLL" int))
(define CRYPT-RANDOM-SLOWPOLL (foreign-value "CRYPT_RANDOM_SLOWPOLL" int))
(define CRYPT-CURSOR-FIRST (foreign-value "CRYPT_CURSOR_FIRST" int))
(define CRYPT-CURSOR-PREVIOUS (foreign-value "CRYPT_CURSOR_PREVIOUS" int))
(define CRYPT-CURSOR-NEXT (foreign-value "CRYPT_CURSOR_NEXT" int))
(define CRYPT-CURSOR-LAST (foreign-value "CRYPT_CURSOR_LAST" int))
(define CRYPT-KEYOPT-NONE (foreign-value "CRYPT_KEYOPT_NONE" int))
(define CRYPT-KEYOPT-READONLY (foreign-value "CRYPT_KEYOPT_READONLY" int))
(define CRYPT-KEYOPT-CREATE (foreign-value "CRYPT_KEYOPT_CREATE" int))
(define CRYPT-KEYOPT-LAST (foreign-value "CRYPT_KEYOPT_LAST" int))
(define CRYPT-OK (foreign-value "CRYPT_OK" int))
(define CRYPT-ERROR-PARAM1 (foreign-value "CRYPT_ERROR_PARAM1" int))
(define CRYPT-ERROR-PARAM2 (foreign-value "CRYPT_ERROR_PARAM2" int))
(define CRYPT-ERROR-PARAM3 (foreign-value "CRYPT_ERROR_PARAM3" int))
(define CRYPT-ERROR-PARAM4 (foreign-value "CRYPT_ERROR_PARAM4" int))
(define CRYPT-ERROR-PARAM5 (foreign-value "CRYPT_ERROR_PARAM5" int))
(define CRYPT-ERROR-PARAM6 (foreign-value "CRYPT_ERROR_PARAM6" int))
(define CRYPT-ERROR-PARAM7 (foreign-value "CRYPT_ERROR_PARAM7" int))
(define CRYPT-ERROR-MEMORY (foreign-value "CRYPT_ERROR_MEMORY" int))
(define CRYPT-ERROR-NOTINITED (foreign-value "CRYPT_ERROR_NOTINITED" int))
(define CRYPT-ERROR-INITED (foreign-value "CRYPT_ERROR_INITED" int))
(define CRYPT-ERROR-NOSECURE (foreign-value "CRYPT_ERROR_NOSECURE" int))
(define CRYPT-ERROR-RANDOM (foreign-value "CRYPT_ERROR_RANDOM" int))
(define CRYPT-ERROR-FAILED (foreign-value "CRYPT_ERROR_FAILED" int))
(define CRYPT-ERROR-NOTAVAIL (foreign-value "CRYPT_ERROR_NOTAVAIL" int))
(define CRYPT-ERROR-PERMISSION (foreign-value "CRYPT_ERROR_PERMISSION" int))
(define CRYPT-ERROR-WRONGKEY (foreign-value "CRYPT_ERROR_WRONGKEY" int))
(define CRYPT-ERROR-INCOMPLETE (foreign-value "CRYPT_ERROR_INCOMPLETE" int))
(define CRYPT-ERROR-COMPLETE (foreign-value "CRYPT_ERROR_COMPLETE" int))
(define CRYPT-ERROR-TIMEOUT (foreign-value "CRYPT_ERROR_TIMEOUT" int))
(define CRYPT-ERROR-INVALID (foreign-value "CRYPT_ERROR_INVALID" int))
(define CRYPT-ERROR-SIGNALLED (foreign-value "CRYPT_ERROR_SIGNALLED" int))
(define CRYPT-ERROR-OVERFLOW (foreign-value "CRYPT_ERROR_OVERFLOW" int))
(define CRYPT-ERROR-UNDERFLOW (foreign-value "CRYPT_ERROR_UNDERFLOW" int))
(define CRYPT-ERROR-BADDATA (foreign-value "CRYPT_ERROR_BADDATA" int))
(define CRYPT-ERROR-SIGNATURE (foreign-value "CRYPT_ERROR_SIGNATURE" int))
(define CRYPT-ERROR-OPEN (foreign-value "CRYPT_ERROR_OPEN" int))
(define CRYPT-ERROR-READ (foreign-value "CRYPT_ERROR_READ" int))
(define CRYPT-ERROR-WRITE (foreign-value "CRYPT_ERROR_WRITE" int))
(define CRYPT-ERROR-NOTFOUND (foreign-value "CRYPT_ERROR_NOTFOUND" int))
(define CRYPT-ERROR-DUPLICATE (foreign-value "CRYPT_ERROR_DUPLICATE" int))
(define CRYPT-ENVELOPE-RESOURCE (foreign-value "CRYPT_ENVELOPE_RESOURCE" int))

(define (crypt-status-error? s)
  (not (eq? CRYPT-OK s)))

(define crypt-error-codes
  (list
   (cons CRYPT-OK
	 "No error")
   (cons CRYPT-ERROR-PARAM1
	 "Parameter 1 in function call was invalid")
   (cons CRYPT-ERROR-PARAM2
	 "Parameter 2 in function call was invalid")
   (cons CRYPT-ERROR-PARAM3
	 "Parameter 3 in function call was invalid")
   (cons CRYPT-ERROR-PARAM4
	 "Parameter 4 in function call was invalid")
   (cons CRYPT-ERROR-PARAM5
	 "Parameter 5 in function call was invalid")
   (cons CRYPT-ERROR-PARAM6
	 "Parameter 6 in function call was invalid")
   (cons CRYPT-ERROR-PARAM7
	 "Parameter 7 in function call was invalid")
   (cons CRYPT-ERROR-FAILED
	 "The operation failed")
   (cons CRYPT-ERROR-INITED
	 "An object or attribute has already been initialized")
   (cons CRYPT-ERROR-MEMORY
	 "Insufficient memory resources to perform the operation")
   (cons CRYPT-ERROR-NOSECURE "The operation could not be performed at the requested security level")
   (cons CRYPT-ERROR-NOTINITED
	 "An object or attribute has not been initialized yet")
   (cons CRYPT-ERROR-RANDOM
	 "Insufficient entropy to perform the operation")
   (cons CRYPT-ERROR-COMPLETE
	 "This operation has been finished and cannot be continued")
   (cons CRYPT-ERROR-INCOMPLETE
	 "An operation requiring several steps has not been completed yet")
   (cons CRYPT-ERROR-INVALID
	 "The key context, certificate or attribute is invalid for this type of operation")
   (cons CRYPT-ERROR-NOTAVAIL
	 "The requested operation cannot be applied to this object")
   (cons CRYPT-ERROR-PERMISSION
	 "You don't have permission to perform this type of operation")
   (cons CRYPT-ERROR-SIGNALLED
	 "An object has been rendered useless due to an external event.")
   (cons CRYPT-ERROR-TIMEOUT
	 "The operation timed out")
   (cons CRYPT-ERROR-WRONGKEY
	 "The key used for the decrypt or verify operation is incorrect")
   (cons CRYPT-ERROR-BADDATA
	 "The data item was corrupt or not all of the data needed to process it was present")
   (cons CRYPT-ERROR-OVERFLOW
	 "There is too much data for this function to work with")
   (cons CRYPT-ERROR-SIGNATURE
	 "The signature or integrity check did not match the data")
   (cons CRYPT-ERROR-UNDERFLOW
	 "There is too little data in the envelope or session for cryptlib to process")
   (cons CRYPT-ERROR-DUPLICATE
	 "The given item is already present in the container object")
   (cons CRYPT-ERROR-NOTFOUND
	 "The requested item is not present in the container object")
   (cons CRYPT-ERROR-OPEN
	 "The container object could not be opened")
   (cons CRYPT-ERROR-READ
	 "The requested item could not be read from the container object")
   (cons CRYPT-ERROR-WRITE
	 "The item could not be written to the container object or the data could not be updated")
   (cons CRYPT-ENVELOPE-RESOURCE
	 "A relevant resource must be added to the envelope before the data in it can be processed")))

(define (crypt-error s)
  (abort
   (make-composite-condition
    (make-property-condition 'exn 'message
			     (let ((msg (assq s crypt-error-codes)))
			       (if msg (cdr msg) "Unknown error")))
    (make-property-condition 'crypt 'code s))))

(define (data-byte-length loc data)
	(cond
		((string? data) (string-length data))
		((u32vector? data) (* 4 (u32vector-length data)))
		((byte-vector? data) (byte-vector-length data))
		(else (error loc
			"unknown data type, cannot obtain length" data))))

(define (crypt-add-cert-extension certificate oid critical?
				  extension #!optional (extension-length #f))
  (let* ((f (foreign-lambda int "cryptAddCertExtension"
			    int byte-vector bool pointer int))
	 (s (cond
	     (extension-length
	      (f certificate oid critical?
		 extension extension-length))
	     ((string? extension)
	      (f certificate oid critical?
		 extension (string-length extension)))
	     ((byte-vector? extension)
	      (f certificate oid critical?
		 extension (byte-vector-length extension)))
	     (else
	      (crypt-error CRYPT-ERORR-PARAM3)))))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-add-private-key keyset key password)
  (let ((s ((foreign-lambda int "cryptAddPrivateKey" int int c-string)
	    keyset key password)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-add-public-key keyset certificate)
  (let ((s ((foreign-lambda int "cryptAddPublicKey" int int)
	    keyset certificate)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-add-random random-data #!optional (random-data-length #f))
  (let* ((f (foreign-lambda int "cryptAddRandom" pointer int))
	 (s (cond
	     (random-data-length
	      (f random-data random-data-length))
	     ((string? random-data)
	      (f random-data (string-length random-data)))
	     ((byte-vector? random-data)
	      (f random-data (byte-vector-length random-data)))
	     (else
	      (crypt-error CRYPT-ERROR-PARAM1)))))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-async-cancel object)
  (let ((s ((foreign-lambda int "cryptAsyncCancel" int) object)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

;;; crypt-async-query returns #t if an asynchronous operation is
;;; running, #f otherwise
(define (crypt-async-query object)
  (let ((s ((foreign-lambda int "cryptAsyncQuery" int) object)))
    (cond
     ((eq? s CRYPT-ERROR-TIMEOUT)
      #t)
     ((crypt-status-error? s)
      (crypt-error s))
     (else
      #f))))

(define (crypt-ca-add-item keyset certificate)
  (let ((s ((foreign-lambda int "cryptCAAddItem" int int)
	    keyset certificate)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

;;; crypt-ca-cert-management gets one parameter less than the normal
;;; cryptlib function call, but an additional optional parameter
;;; indicates whether the first argument of the cryptlib call should
;;; be used (and the retrieved value returned) or not.
(define (crypt-ca-cert-management action keyset ca-key cert-request
				  #!optional (get-cert? #f))
  (let* ((cert (if get-cert? (make-u32vector 1) #f))
	 (s ((foreign-lambda int "cryptCACertManagement"
			     u32vector int int int int)
	     cert action keyset ca-key cert-request)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(if get-cert? (u32vector-ref cert 0) (void)))))

;;; crypt-ca-get-item returns the result retrieved in the second
;;; parameter to the cryptlib call
(define (crypt-ca-get-item keyset cert-type id-type id)
  (let* ((cert (make-u32vector 1))
	 (s ((foreign-lambda int "cryptCAGetItem" int u32vector int
			     int c-string)
	     keyset cert cert-type id-type id)))
    (if (crypt-status-error? s) (crypt-error s) (u32vector-ref cert 0))))

(define (crypt-check-cert certificate sig-check-key)
  (let ((s ((foreign-lambda int "cryptCheckCert" int int)
	    certificate sig-check-key)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-check-signature signature sig-check-key hash-context)
	(let* (	[sig-len (data-byte-length 'crypt-check-signature signature)]
			[s ((foreign-lambda int "cryptCheckSignature" pointer int int int)
				signature sig-len sig-check-key hash-context)])
		(if (crypt-status-error? s) (crypt-error s) (void))))

;;; crypt-check-signature-ex returns the result retrieved in the last
;;; parameter to the cryptlib call if its optional argument is true
(define (crypt-check-signature-ex signature sig-check-key hash-context #!optional get-extra-data?)
	(let* (	[sig-len (data-byte-length 'crypt-check-signature-ex signature)]
			[extra-data (if get-extra-data? (make-u32vector 1) #f)]
			[s ((foreign-lambda int "cryptCheckSignatureEx" pointer int int int u32vector)
					signature sig-len sig-check-key hash-context extra-data)])
		(if (crypt-status-error? s)
			(crypt-error s)
			(if get-extra-data? (u32vector-ref extra-data 0) (void)))))

;;; crypt-create-cert returns the result retrieved in the first
;;; parameter to the cryptlib call
(define (crypt-create-cert user cert-type)
  (let* ((cert (make-u32vector 1))
	 (s ((foreign-lambda int "cryptCreateCert" u32vector int int)
	     cert user cert-type)))
    (if (crypt-status-error? s) (crypt-error s) (u32vector-ref cert 0))))

;;; crypt-create-context returns the result retrieved in the first
;;; parameter to the cryptlib call
(define (crypt-create-context user algo)
  (let* ((context (make-u32vector 1))
	 (s ((foreign-lambda int "cryptCreateContext"
			     u32vector int int)
	     context user algo)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref context 0))))

;;; crypt-create-envelope returns the result retrieved in the first
;;; parameter to the cryptlib call
(define (crypt-create-envelope user format-type)
  (let* ((envelope (make-u32vector 1))
	 (s ((foreign-lambda int "cryptCreateEnvelope"
			     u32vector int int)
	     envelope user format-type)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref envelope 0))))

;;; crypt-create-session returns the result retrieved in the first
;;; parameter to the cryptlib call
(define (crypt-create-session user session-type)
  (let* ((session (make-u32vector 1))
	 (s ((foreign-lambda int "cryptCreateSession"
			     u32vector int int)
	     session user session-type)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref session 0))))

;;; crypt-create-signature returns the result retrieved in the first
;;; two parameters to the cryptlib call as a string
(define (crypt-create-signature sign-context hash-context)
	(let ([f (foreign-lambda int "cryptCreateSignature"
				pointer int u32vector int int)])
		(let* (	[size (make-u32vector 1)]
				[s (f #f 4 size sign-context hash-context)])
		(if (crypt-status-error? s) (crypt-error s))
		(let (	[bufsiz (u32vector-ref size 0)]
				[signature (make-string bufsiz)])
			(set! s (f signature bufsiz size sign-context hash-context))
			(if (crypt-status-error? s) (crypt-error s) signature)))))

;;; crypt-create-signature-ex returns the result retrieved in the first
;;; parameter to the cryptlib call as a string
(define (crypt-create-signature-ex format-type sign-context hash-context extra-data)
	(let ([f (foreign-lambda int "cryptCreateSignatureEx"
				pointer int u32vector int int int int)])
		(let* (	[size (make-u32vector 1)]
				[s (f #f 4 size format-type sign-context hash-context extra-data)])
			(if (crypt-status-error? s) (crypt-error s))
			(let (	[bufsiz (u32vector-ref size 0)]
					[signature (make-string bufsiz)])
				(set! s (f signature bufsiz size format-type sign-context hash-context extra-data))
				(if (crypt-status-error? s) (crypt-error s) signature)))))

(define (crypt-decrypt context buffer #!optional (buffer-length #f))
  (let* ((f (foreign-lambda int "cryptDecrypt" int pointer int))
	 (s (cond
	     (buffer-length
	      (f context buffer buffer-length))
	     ((string? buffer)
	      (f context buffer (string-length buffer)))
	     ((byte-vector? buffer)
	      (f context buffer (byte-vector-length buffer)))
	     (else
	      (crypt-error CRYPT-ERROR-PARAM2)))))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-delete-attribute object attribute-type)
  (let ((s ((foreign-lambda int "cryptDeleteAttribute" int int)
	    object attribute-type)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-delete-cert-extension certificate oid)
  (let ((s ((foreign-lambda int "cryptDeleteCertExtension"
			    int byte-vector)
	    certificate oid)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-delete-key object keyid-type keyid)
  (let ((s ((foreign-lambda int "cryptDeleteKey" int int c-string)
	    object keyid-type keyid)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-destroy-cert certificate)
  (let ((s ((foreign-lambda int "cryptDestroyCert" int) certificate)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-destroy-context context)
  (let ((s ((foreign-lambda int "cryptDestroyContext" int) context)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-destroy-envelope envelope)
  (let ((s ((foreign-lambda int "cryptDestroyEnvelope" int) envelope)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-destroy-object object)
  (let ((s ((foreign-lambda int "cryptDestroyObject" int) object)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-destroy-session session)
  (let ((s ((foreign-lambda int "cryptDestroySession" int) session)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-device-close device)
  (let ((s ((foreign-lambda int "cryptDeviceClose" int) device)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

;;; crypt-device-create-context returns the result retrieved in the
;;; second parameter to the cryptlib call
(define (crypt-device-create-context device algo)
  (let* ((context (make-u32vector 1))
	 (s ((foreign-lambda int "cryptDeviceCreateContext"
			     int u32vector int)
	     device context algo)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref context 0))))

;;; crypt-device-open returns the result retrieved in the first
;;; parameter to the cryptlib call
(define (crypt-device-open user device-type name)
  (let* ((device (make-u32vector 1))
	  (s ((foreign-lambda int "cryptDeviceOpen"
			      u32vector int int c-string)
	      device user device-type name)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref device 0))))

#>
static char *crypt_query_capability(int device, int algo, int *params) {
  char *name;
  CRYPT_QUERY_INFO qi;

  if (device != CRYPT_UNUSED)
    params[0] = cryptDeviceQueryCapability(device, algo, &qi);
  else
    params[0] = cryptQueryCapability(algo, &qi);

  params[1] = qi.blockSize;
  params[2] = qi.minKeySize;
  params[3] = qi.keySize;
  params[4] = qi.maxKeySize;

  name = (char *)malloc(strlen(qi.algoName));
  if (name != NULL)
    return strcpy(name, qi.algoName);
  else
    return NULL;
}
<#

;;; crypt-device-query-capability returns the following values
;;; - name of the algorithm
;;; - block size
;;; - minimum key size
;;; - standard key size
;;; - maximum key size
(define (crypt-device-query-capability device algo)
  (let* ((params (make-u32vector 5))
	 (name ((foreign-lambda c-string* "crypt_query_capability"
				int int u32vector)
		device algo params)))
    (if (crypt-status-error? (u32vector-ref params 0))
	(crypt-error (u32vector-ref params 0))
	(values
	 name
	 (u32vector-ref params 1)
	 (u32vector-ref params 2)
	 (u32vector-ref params 3)
	 (u32vector-ref params 4)))))

(define (crypt-encrypt context buffer #!optional (buffer-length #f))
  (let* ((f (foreign-lambda int "cryptEncrypt" int pointer int))
	 (s (cond
	     (buffer-length
	      (f context buffer buffer-length))
	     ((string? buffer)
	      (f context buffer (string-length buffer)))
	     ((byte-vector? buffer)
	      (f context buffer (byte-vector-length buffer)))
	     (else
	      (crypt-error CRYPT-ERROR-PARAM2)))))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-end)
  (let ((s ((foreign-lambda int "cryptEnd"))))
    (if (crypt-status-error? s) (crypt-error s) (void))))

;;; crypt-export-cert returns the result retrieved in the first two
;;; parameters to the cryptlib call as a string
(define (crypt-export-cert cert-format-type certificate)
	(let ([f (foreign-lambda int "cryptExportCert"
				pointer int u32vector int int)])
		(let* (	[size (make-u32vector 1)]
				[s (f #f 4 size cert-format-type certificate)])
			(if (crypt-status-error? s) (crypt-error s))
			(let (	[bufsiz (u32vector-ref size 0)]
					[buffer (make-string bufsiz)])
				(set! s (f buffer bufsiz size cert-format-type certificate))
				(if (crypt-status-error? s) (crypt-error s)) buffer))))

;;; crypt-export-key returns the result retrieved in the first two
;;; parameters to the cryptlib call as a string
(define (crypt-export-key export-key session-key-context)
	(let ([f (foreign-lambda int "cryptExportKey" pointer int u32vector int int)])
		(let* (	[size (make-u32vector 1)]
				[s (f #f 4 size export-key session-key-context)])
		(if (crypt-status-error? s) (crypt-error s))
		(let* (	[bufsiz (u32vector-ref size 0)]
				[buffer (make-string bufsiz)])
			(set! s (f buffer bufsiz size export-key session-key-context))
			(if (crypt-status-error? s) (crypt-error s))
			buffer))))

;;; crypt-export-key-ex returns the result retrieved in the first two
;;; parameters to the cryptlib call as a string
(define (crypt-export-key-ex format-type export-key session-key-context)
	(let ([f (foreign-lambda int "cryptExportKeyEx"
				pointer int u32vector int int int)])
		(let* (	[size (make-u32vector 1)]
				[s (f #f 4 size format-type export-key session-key-context)])
			(if (crypt-status-error? s) (crypt-error s))
			(let* (	[bufsiz (u32vector-ref size 0)]
					[buffer (make-string bufsiz)])
				(set! s (f buffer bufsiz size format-type export-key session-key-context))
				(if (crypt-status-error? s) (crypt-error s))
				buffer))))

(define (crypt-flush-data handle)
  (let ((s ((foreign-lambda int "cryptFlushData" int) handle)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-generate-key context)
  (let ((s ((foreign-lambda int "cryptGenerateKey" int) context)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-generate-key-async context)
  (let ((s ((foreign-lambda int "cryptGenerateKeyAsync" int) context)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

;;; crypt-get-attribute returns the result retrieved in the last
;;; parameter to the cryptlib call
(define (crypt-get-attribute object attribute-type)
  (let* ((attr (make-u32vector 1))
	 (s ((foreign-lambda int "cryptGetAttribute"
			     int int u32vector)
	     object attribute-type attr)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref attr 0))))

;;; crypt-get-attribute-string returns the result retrieved in the
;;; last two parameters to the cryptlib call as a string
(define (crypt-get-attribute-string object attribute-type)
  (let ((f (foreign-lambda int "cryptGetAttributeString"
			   int int pointer u32vector)))
    (let* ((size (make-u32vector 1))
	   (s (f object attribute-type #f size)))
      (if (crypt-status-error? s) (crypt-error s))
      (let ((attr (make-string (u32vector-ref size 0))))
	(set! s (f object attribute-type attr size))
	(if (crypt-status-error? s) (crypt-error s)) attr))))

;;; crypt-get-cert-extension returns the data retrieved in the last
;;; two parameters to the cryptlib call as a string and the
;;; critical flag as a boolean
(define (crypt-get-cert-extension certificate oid)
	(let ([f (foreign-lambda int "cryptGetCertExtension"
					int byte-vector u32vector pointer int u32vector)])
		(let* (	[size (make-u32vector 1)]
				[critical? (make-u32vector 1)]
				[s (f certificate oid critical? #f 4 size)])
			(if (crypt-status-error? s) (crypt-error s))
			(let* (	[bufsiz (u32vector-ref size 0)]
					[buffer (make-string bufsiz)])
				(set! s (f certificate oid critical? buffer bufsiz size))
				(if (crypt-status-error? s)
					(crypt-error s)
					(values buffer (not (zero? (u32vector-ref critical? 0)))))))))

;;; crypt-get-private-key returns the data retrieved in the 3rd
;;; parameter to the cryptlib call
(define (crypt-get-private-key handle keyid-type keyid password)
  (let* ((context (make-u32vector 1))
	 (s ((foreign-lambda int "cryptGetPrivateKey"
			     int u32vector int c-string c-string)
	     handle context keyid-type keyid password)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref context 0))))

;;; crypt-get-publib-key returns the data retrieved in the second
;;; parameter to the cryptlib call
(define (crypt-get-public-key object keyid-type keyid)
  (let* ((handle (make-u32vector 1))
	 (s ((foreign-lambda int "cryptGetPublicKey"
			     int u32vector int c-string)
	     object handle keyid-type keyid)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref handle 0))))

;;; crypt-import-cert returns the data retrieved in the last parameter
;;; to the cryptlib call. Also note that the parameters have been
;;; reordered (user is the first now) to let the length parameter be
;;; optional
(define (crypt-import-cert user cert-data
			   #!optional (cert-data-length #f))
  (let* ((f (foreign-lambda int "cryptImportCert"
			    pointer int int u32vector))
	 (certificate (make-u32vector 1))
	 (s (cond
	     (cert-data-length
	      (f cert-data cert-data-length user certificate))
	     ((string? cert-data)
	      (f cert-data (string-length cert-data) user certificate))
	     ((byte-vector? cert-data)
	      (f cert-data (byte-vector-length cert-data) user
		 certificate))
	     (else
	      (crypt-error CRYPT-ERROR-PARAM2)))))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref certificate 0))))

(define (crypt-import-key key-data import-context session-key-context)
	(let* (	[key-data-len (data-byte-length 'crypt-import-key key-data)]
			[s ((foreign-lambda int "cryptImportKey" pointer int int int)
					key-data key-data-len import-context session-key-context)])
		(if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-init)
  (let ((s ((foreign-lambda int "cryptInit"))))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-keyset-close keyset)
  (let ((s ((foreign-lambda int "cryptKeysetClose" int) keyset)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

;;; crypt-keyset-open returns the data retrieved in the first
;;; parameter to the cryptlib call.
(define (crypt-keyset-open user keyset-type keyset-name keyset-options)
  (let* ((keyset (make-u32vector 1))
	 (s ((foreign-lambda int "cryptKeysetOpen"
			     u32vector int int c-string int)
	     keyset user keyset-type keyset-name keyset-options)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref keyset 0))))

;;; crypt-pop-data returns the buffer and the number of bytes read
;;; into it. also note that the sequence (and number) of parameters is
;;; different from the original cryptlib version
(define (crypt-pop-data handle max-length #!optional (buffer #f))
  (let* ((length (make-u32vector 1))
	 (s (begin
	      (unless buffer (set! buffer (make-string max-length)))
	      ((foreign-lambda int "cryptPopData"
			       int pointer int u32vector)
	       handle buffer max-length length))))
    (if (crypt-status-error? s)
	(crypt-error s)
	(list buffer (u32vector-ref length 0)))))

;;; crypt-push-data returns the number of bytes copied from the buffer
(define (crypt-push-data handle buffer #!optional (buffer-length #f))
  (let* ((f (foreign-lambda int "cryptPushData"
			    int pointer int u32vector))
	 (length (make-u32vector 1))
	 (s (cond
	     (buffer-length
	      (f handle buffer buffer-length length))
	     ((string? buffer)
	      (f handle buffer (string-length buffer) length))
	     ((byte-vector? buffer)
	      (f handle buffer (byte-vector-length buffer) length))
	     (else
	      (crypt-error CRYPT-ERROR-PARAM2)))))
    (if (crypt-status-error? s)
	(crypt-error s)
	(u32vector-ref length 0))))

;;; crypt-query-capability returns the following values
;;; - name of the algorithm
;;; - block size
;;; - minimum key size
;;; - standard key size
;;; - maximum key size
(define (crypt-query-capability algo)
  (let* ((params (make-u32vector 5))
	 (name ((foreign-lambda c-string* "crypt_query_capability"
				int int u32vector)
		CRYPT-UNUSED algo params)))
    (if (crypt-status-error? (u32vector-ref params 0))
	(crypt-error (u32vector-ref params 0))
	(values
	 name
	 (u32vector-ref params 1)
	 (u32vector-ref params 2)
	 (u32vector-ref params 3)
	 (u32vector-ref params 4)))))

#>
static int crypt_query_object(void *object, int *params, char *salt) {
  int s;
  CRYPT_OBJECT_INFO oi;

  s = cryptQueryObject(object, sizeof(oi), &oi);

  params[0] = oi.objectType;
  params[1] = oi.cryptAlgo;
  params[2] = oi.cryptMode;
  params[3] = oi.hashAlgo;
  params[4] = oi.saltSize;

  memcpy(salt, oi.salt, CRYPT_MAX_HASHSIZE);

  return s;
}
<#

;;; crypt-query-object returns the following values
;;; - the object type
;;; - the encryption algorithm
;;; - the encryption mode
;;; - the hash algorithm
;;; - the salt for derived keys
;;; - the size of the salt
(define (crypt-query-object object)
  (let* ((params (make-u32vector 5))
	 (salt (make-string CRYPT-MAX-HASHSIZE))
	 (s ((foreign-lambda int "crypt_query_object"
			     pointer u32vector pointer)
	     object params salt)))
    (if (crypt-status-error? s)
	(crypt-error s)
	(values
	 (u32vector-ref params 0)
	 (u32vector-ref params 1)
	 (u32vector-ref params 2)
	 (u32vector-ref params 3)
	 (u32vector-ref params 4)
	 salt))))

(define (crypt-set-attribute object attribute-type attr)
  (let ((s ((foreign-lambda int "cryptSetAttribute" int int int)
	    object attribute-type attr)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-set-attribute-string object attribute-type attr
				    #!optional (attr-length #f))
  (let* ((f (foreign-lambda int "cryptSetAttributeString"
			    int int pointer int))
	 (s (cond
	     (attr-length
	      (f object attribute-type attr attr-length))
	     ((string? attr)
	      (f object attribute-type attr (string-length attr)))
	     ((byte-vector? attr)
	      (f object attribute-type attr
		 (byte-vector-length attr)))
	     (else
	      (crypt-error CRYPT-ERROR-PARAM3)))))
    (if (crypt-status-error? s) (crypt-error s) (void))))

(define (crypt-sign-cert certificate sign-context)
  (let ((s ((foreign-lambda int "cryptSignCert" int int)
	    certificate sign-context)))
    (if (crypt-status-error? s) (crypt-error s) (void))))

;;; crypt-input-port returns an input port which wraps calls to
;;; crypt-pop-data. closing the port does NOT destroy the underlying
;;; cryptlib object
(define (crypt-input-port object)
  (let ((chbuf (make-string 1))
	(filled? #f))
    (make-input-port
     ;; read
     (lambda ()
       (if filled?
	   (begin
	     (set! filled? #f)
	     (string-ref chbuf 0))
	   (if (> (cadr (crypt-pop-data object 1 chbuf)) 0)
	       (string-ref chbuf 0)
	       #!eof)))
     ;; ready?
     (lambda ()
       (if filled?
	   #t
	   (if (> (cadr (crypt-pop-data object 1 chbuf)) 0)
	       (begin
		 (set! filled? #t)
		 #t)
	       #f)))
     ;; close
     void
     ;; peek
     (lambda ()
       (if filled?
	   (string-ref chbuf 0)
	   (if (> (cadr (crypt-pop-data object 1 chbuf)) 0)
	       (begin
		 (set! filled? #t)
		 (string-ref chbuf 0))
	       #!eof))))))

;;; crypt-output-port returns an output port which wraps calls to
;;; crypt-push-data. closing the port does NOT destroy the underlying
;;; cryptlib object
(define (crypt-output-port object)
  (make-output-port
   ;; write
   (lambda (data)
     (crypt-push-data object data (##sys#size data)))
   ;; close
   void
   ;; flush
   (lambda ()
     (crypt-flush-data object))))
